@@ -1,7 +1,7 @@
 // callbacks.h
 // LiVES
-// (c) G. Finch 2003 <salsaman@xs4all.nl>
-// Released under the GPL 2.0 or higher
+// (c) G. Finch <salsaman@xs4all.nl> 2003 - 2009
+// Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
 #include <gtk/gtk.h>
@@ -144,9 +144,6 @@ void
 on_rev_clipboard_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void 
-on_vidp_entry_changed (GtkWidget *vidp_entry, gpointer);
-
 void 
 on_audp_entry_changed (GtkWidget *vidp_entry, gpointer);
 
@@ -175,6 +172,30 @@ void
 on_about_activate                     (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
+void
+show_manual_activate                     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+email_author_activate                     (GtkMenuItem     *menuitem,
+					   gpointer         user_data);
+
+void
+donate_activate                     (GtkMenuItem     *menuitem,
+					   gpointer         user_data);
+
+void
+report_bug_activate                     (GtkMenuItem     *menuitem,
+					   gpointer         user_data);
+
+void
+suggest_feature_activate                     (GtkMenuItem     *menuitem,
+					      gpointer         user_data);
+
+void
+help_translate_activate                     (GtkMenuItem     *menuitem,
+					   gpointer         user_data);
+
 void
 on_ok_button1_clicked                  (GtkButton       *button,
                                         gpointer         user_data);
@@ -192,6 +213,10 @@ void
 on_locbw_toggled                (GtkToggleButton *togglebutton,
 				 gpointer         user_data);
 
+void
+on_opendeint_toggled                (GtkToggleButton *togglebutton,
+				 gpointer         user_data);
+
 void
 on_autoreload_toggled                (GtkToggleButton *togglebutton,
 				 gpointer         user_data);
@@ -205,10 +230,16 @@ void
 on_open_sel_ok_button_clicked                  (GtkButton       *button,
                                         gpointer         user_data);
 
+
+void on_save_textview_clicked (GtkButton *button, gpointer user_data);
+
 void
 on_cancel_button1_clicked              (GtkButton       *button,
                                         gpointer         user_data);
 
+gboolean on_cancel_button1_clicked_del(GtkWidget *widget, GdkEvent *event, gpointer user_data);
+
+
 void
 on_button3_clicked                     (GtkButton       *button,
                                         gpointer         user_data);
@@ -276,6 +307,10 @@ void
 on_info_ok_button_clicked2            (GtkButton       *button,
 				      gpointer         user_data);
 
+void
+on_details_button_clicked            (GtkButton       *button,
+				      gpointer         user_data);
+
 
 void
 on_resize_activate                    (GtkMenuItem     *menuitem,
@@ -387,6 +422,11 @@ void
 changed_fps_during_pb           (GtkSpinButton   *spinbutton,
 				 gpointer         user_data);
 
+gboolean
+on_mouse_scroll           (GtkWidget       *widget,
+			   GdkEventScroll  *event,
+			   gpointer         user_data);
+
 gboolean
 on_mouse_sel_update           (GtkWidget       *widget,
 			       GdkEventMotion  *event,
@@ -434,8 +474,8 @@ void
 on_capture_activate                (GtkMenuItem     *menuitem,
 				    gpointer         user_data);
 
-gboolean 
-on_capture2_activate(gpointer data);
+void 
+on_capture2_activate(void);
 
 
 void
@@ -486,6 +526,8 @@ void
 on_record_perf_activate                      (GtkMenuItem     *menuitem,
 					      gpointer         user_data);
 
+gboolean record_toggle_callback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data);
+
 gboolean dirchange_callback (GtkAccelGroup *, GObject *, guint, GdkModifierType, gpointer user_data); // defined as extern in dialogs.c
 
 gboolean fps_reset_callback (GtkAccelGroup *, GObject *, guint, GdkModifierType, gpointer user_data);
@@ -493,9 +535,6 @@ gboolean fps_reset_callback (GtkAccelGroup *, GObject *, guint, GdkModifierType,
 
 gboolean mute_audio_callback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer);
 
-void
-on_reorder_activate                (GtkMenuItem     *menuitem,
-				    gpointer         user_data);
 
 void
 on_xmms_random_audio_activate                (GtkMenuItem     *menuitem,
@@ -506,6 +545,9 @@ on_stop_activate_by_del                  (GtkWidget       *widget,
 					  GdkEvent        *event,
 					  gpointer         user_data);
 
+void on_pause_clicked(void);
+
+
 void
 on_select_start_only_activate                (GtkMenuItem     *menuitem,
 					      gpointer         user_data);
@@ -561,10 +603,15 @@ on_recaudsel_activate (GtkMenuItem     *menuitem,
 void
 on_recaudclip_ok_clicked                      (GtkButton *button,
 					       gpointer user_data);
-
+#ifdef HAVE_GTK_NICE_VERSION
+void
+on_volume_slider_value_changed           (GtkScaleButton   *sbutton,
+					  gpointer user_data);
+#else
 void
 on_volume_slider_value_changed           (GtkRange   *slider,
 					  gpointer user_data);
+#endif
 
 void
 on_fade_audio_activate (GtkMenuItem     *menuitem,
@@ -744,3 +791,5 @@ void popup_lmap_errors(GtkMenuItem *, gpointer);
 void on_lerrors_close_clicked (GtkButton *, gpointer);
 void on_lerrors_clear_clicked (GtkButton *, gpointer);
 void on_lerrors_delete_clicked (GtkButton *, gpointer);
+
+void on_init_aplayer_toggled (GtkToggleButton *, gpointer new_player);
