@@ -1,23 +1,19 @@
 // framedraw.c
 // LiVES
 // (c) G. Finch (salsaman@xs4all.nl)
-// see file COPYING for licensing details : released under the GNU GPL 2.0 or higher
+// see file COPYING for licensing details : released under the GNU GPL 3 or later
 
 // functions for the 'framedraw' widget - lets users draw on frames :-)
+#include "../libweed/weed.h"
+#include "../libweed/weed-host.h"
+#include "../libweed/weed-effects.h"
 
 #include "main.h"
 #include "callbacks.h"
 #include "support.h"
 #include "interface.h"
 #include "effects.h"
-
-
-#define MIN_PRE_X 58
-#define MIN_PRE_Y 32
-
-#define MAX_PRE_X 320
-#define MAX_PRE_Y 240
-
+#include "cvirtual.h"
 
 // set by mouse button press
 static gint xstart,ystart;
@@ -25,20 +21,65 @@ static gboolean b1_held;
 
 static gint fdwidth,fdheight;
 
-void start_preview (GtkButton *button, gpointer rfx) {
-  gchar *com=g_strdup_printf("smogrify cancel_hup %s 1",cfile->handle);
-  system(com);
+static gdouble calc_fd_scale(gint width, gint height) {
+  gdouble scale=1.;
+
+ if (width<MIN_PRE_X) {
+    width=MIN_PRE_X;
+  }
+  if (height<MIN_PRE_Y) {
+    height=MIN_PRE_Y;
+  }
+
+  if (width>MAX_PRE_X) scale=(gdouble)width/(gdouble)MAX_PRE_X;
+  if (height>MAX_PRE_Y&&(height/MAX_PRE_Y>scale)) scale=(gdouble)height/(gdouble)MAX_PRE_Y;
+  return scale;
+
+}
+
+static void start_preview (GtkButton *button, lives_rfx_t *rfx) {
+  int i;
+  gchar *com=g_strdup_printf("smogrify stopsubsub %s 2>/dev/null",cfile->handle);
+
+  gtk_widget_set_sensitive(mainw->framedraw_preview,FALSE);
+  while (g_main_context_iteration(NULL,FALSE));
+
+  if (mainw->did_rfx_preview) {
+    dummyvar=system(com);
+    do_rfx_cleanup(rfx);
+  }
+
   g_free(com);
+
+  for (i=0;i<rfx->num_params;i++) {
+    rfx->params[i].changed=FALSE;
+  }
+
   do_effect(rfx,TRUE); // actually start effect processing in the background
+
   gtk_widget_set_sensitive(mainw->framedraw_spinbutton,TRUE);
   gtk_widget_set_sensitive(mainw->framedraw_scale,TRUE);
-  if (mainw->framedraw_frame>cfile->start) gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->framedraw_spinbutton),cfile->start);
-  else load_rfx_preview();
+
+  if (mainw->framedraw_frame>cfile->start&&!(cfile->start==0&&mainw->framedraw_frame==1)) gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->framedraw_spinbutton),cfile->start);
+  else load_rfx_preview(rfx);
   mainw->did_rfx_preview=TRUE;
 }
 
 
-void framedraw_connect(lives_special_framedraw_rect_t *framedraw, gint width, gint height) {
+
+
+void framedraw_connect_spinbutton(lives_special_framedraw_rect_t *framedraw, lives_rfx_t *rfx) {
+  framedraw->rfx=rfx;
+
+  g_signal_connect_after (GTK_OBJECT (mainw->framedraw_spinbutton), "value_changed",
+			  G_CALLBACK (after_framedraw_frame_spinbutton_changed),
+			  framedraw);
+
+}
+
+
+
+void framedraw_connect(lives_special_framedraw_rect_t *framedraw, gint width, gint height, lives_rfx_t *rfx) {
   mainw->framedraw_bitmap = gdk_pixmap_new (NULL, width, height, 1);
   if (mainw->framedraw_bitmap!=NULL) mainw->framedraw_bitmapgc=gdk_gc_new (mainw->framedraw_bitmap);
 
@@ -62,10 +103,7 @@ void framedraw_connect(lives_special_framedraw_rect_t *framedraw, gint width, gi
   g_signal_connect (GTK_OBJECT(mainw->framedraw), "enter-notify-event",G_CALLBACK (on_framedraw_enter),framedraw);
   g_signal_connect (GTK_OBJECT(mainw->framedraw), "leave-notify-event",G_CALLBACK (on_framedraw_leave),framedraw);
 
-
-  g_signal_connect_after (GTK_OBJECT (mainw->framedraw_spinbutton), "value_changed",
-			  G_CALLBACK (after_framedraw_frame_spinbutton_changed),
-			  framedraw);
+  framedraw_connect_spinbutton(framedraw,rfx);
 
   gtk_widget_modify_bg (mainw->fd_frame, GTK_STATE_NORMAL, &palette->light_red);
   framedraw_redraw(framedraw, TRUE, NULL);
@@ -102,6 +140,7 @@ void framedraw_add_reset(GtkVBox *box, lives_special_framedraw_rect_t *framedraw
 
 
 
+
 void
 widget_add_framedraw (GtkVBox *box, gint start, gint end, gboolean add_preview_button, gint width, gint height) {
   // adds the frame draw widget to box
@@ -117,7 +156,8 @@ widget_add_framedraw (GtkVBox *box, gint start, gint end, gboolean add_preview_b
   GtkObject *spinbutton_adj;
   GtkWidget *label2;
   GtkWidget *frame;
-  gdouble scale=1.;
+ 
+  lives_rfx_t *rfx;
 
   b1_held=FALSE;
 
@@ -131,18 +171,9 @@ widget_add_framedraw (GtkVBox *box, gint start, gint end, gboolean add_preview_b
   gtk_box_pack_start (GTK_BOX (GTK_WIDGET (box)->parent), vbox, FALSE, FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
 
-  if (width<MIN_PRE_X) {
-    width=MIN_PRE_X;
-  }
-  if (height<MIN_PRE_Y) {
-    height=MIN_PRE_Y;
-  }
-
-  if (width>MAX_PRE_X) scale=(gdouble)width/(gdouble)MAX_PRE_X;
-  if (height>MAX_PRE_Y&&(height/MAX_PRE_Y>scale)) scale=(gdouble)height/(gdouble)MAX_PRE_Y;
-  width/=scale;
-  height/=scale;
-  mainw->fd_scale=scale;
+  mainw->fd_scale=calc_fd_scale(width,height);
+  width/=mainw->fd_scale;
+  height/=mainw->fd_scale;
 
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -213,24 +244,25 @@ widget_add_framedraw (GtkVBox *box, gint start, gint end, gboolean add_preview_b
   gtk_scale_set_draw_value(GTK_SCALE(mainw->framedraw_scale),FALSE);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label2),mainw->framedraw_scale);
 
-  if (add_preview_button) {
-    lives_rfx_t *rfx=g_object_get_data(G_OBJECT(gtk_widget_get_toplevel(GTK_WIDGET(box))),"rfx");
-    mainw->framedraw_preview = gtk_button_new_from_stock ("gtk-refresh");
-    gtk_button_set_label (GTK_BUTTON (mainw->framedraw_preview),_ ("_Preview"));
-    gtk_button_set_use_underline (GTK_BUTTON (mainw->framedraw_preview), TRUE);
-    gtk_box_pack_start (GTK_BOX (hbox), mainw->framedraw_preview, TRUE, FALSE, 0);
-    gtk_widget_set_sensitive(mainw->framedraw_spinbutton,FALSE);
-    gtk_widget_set_sensitive(mainw->framedraw_scale,FALSE);
-    g_signal_connect (mainw->framedraw_preview, "clicked",G_CALLBACK (start_preview),rfx);
-  }
-  else mainw->framedraw_preview=NULL;
-
+  rfx=g_object_get_data(G_OBJECT(gtk_widget_get_toplevel(GTK_WIDGET(box))),"rfx");
+  mainw->framedraw_preview = gtk_button_new_from_stock ("gtk-refresh");
+  gtk_button_set_label (GTK_BUTTON (mainw->framedraw_preview),_ ("_Preview"));
+  gtk_button_set_use_underline (GTK_BUTTON (mainw->framedraw_preview), TRUE);
+  gtk_box_pack_start (GTK_BOX (hbox), mainw->framedraw_preview, TRUE, FALSE, 0);
+  gtk_widget_set_sensitive(mainw->framedraw_spinbutton,FALSE);
+  gtk_widget_set_sensitive(mainw->framedraw_scale,FALSE);
+  g_signal_connect (mainw->framedraw_preview, "clicked",G_CALLBACK (start_preview),rfx);
+  
   if (mainw->framedraw_colourgc!=NULL) {
     g_object_unref (mainw->framedraw_colourgc);
     mainw->framedraw_colourgc=NULL;
   }
 
   gtk_widget_show_all (vbox);
+
+  if (!add_preview_button) {
+    gtk_widget_hide(mainw->framedraw_preview);
+  }
 }
 
 
@@ -315,8 +347,8 @@ framedraw_redraw (lives_special_framedraw_rect_t * framedraw, gboolean reload, G
 
     xstart=(gint)(gtk_spin_button_get_value (GTK_SPIN_BUTTON (framedraw->xstart_widget))*outwidth+.4999);
     ystart=(gint)(gtk_spin_button_get_value (GTK_SPIN_BUTTON (framedraw->ystart_widget))*outheight+.4999);
-    xend=xstart+(gint)(gtk_spin_button_get_value (GTK_SPIN_BUTTON (framedraw->xend_widget))*inwidth+.4999);
-    yend=ystart+(gint)(gtk_spin_button_get_value (GTK_SPIN_BUTTON (framedraw->yend_widget))*inheight+.4999);
+    xend=xstart+(gint)(gtk_spin_button_get_value (GTK_SPIN_BUTTON (framedraw->xend_widget))*outwidth+.4999);
+    yend=ystart+(gint)(gtk_spin_button_get_value (GTK_SPIN_BUTTON (framedraw->yend_widget))*outheight+.4999);
 
     gdk_draw_line (GDK_DRAWABLE (mainw->framedraw_bitmap),
 		   mainw->framedraw_bitmapgc,
@@ -384,42 +416,81 @@ framedraw_redraw (lives_special_framedraw_rect_t * framedraw, gboolean reload, G
 
 
 
-void load_rfx_preview(void) {
-  GError *error=NULL;
-  gchar buf[256];
-  GdkPixbuf *pixbuf=NULL,*pixbuf2;
+void load_rfx_preview(lives_rfx_t *rfx) {
+  GdkPixbuf *pixbuf;
   FILE *infofile;
-  int max_frame;
+  int max_frame=0,tot_frames=0;
+  gint current_file=mainw->current_file;
+  weed_timecode_t tc;
+  const gchar *img_ext;
+
+  if (mainw->framedraw_frame==0) mainw->framedraw_frame=1;
+
+  lives_set_cursor_style(LIVES_CURSOR_BUSY,NULL);
+
+  if (cfile->clip_type==CLIP_TYPE_FILE) {
+    // pull some frames to get started
+    gint vend=mainw->framedraw_frame+FX_FRAME_PUMP_VAL;
+    if (vend>cfile->end) vend=cfile->end;
+    virtual_to_images(mainw->current_file,mainw->framedraw_frame,vend);
+  }
 
   clear_mainw_msg();
 
-  while (!(infofile=fopen(cfile->info_file,"r"))); // wait until we get at least 1 frame
-  
-  fgets(mainw->msg,512,infofile);
+  while (!(infofile=fopen(cfile->info_file,"r"))) g_usleep(prefs->sleep_time); // wait until we get at least 1 frame
+
+  dummychar=fgets(mainw->msg,512,infofile);
   fclose(infofile);
 
-  max_frame=atoi(mainw->msg);
-  if (mainw->framedraw_frame>max_frame) {
-    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->framedraw_spinbutton),max_frame);
-    return;
-  }
-
-  g_snprintf(buf,256,"%s/%s/%08d.pre",prefs->tmpdir,cfile->handle,mainw->framedraw_frame);
-  do {
-    if (!(error==NULL)) {
-      g_error_free(error);
-      error=NULL;
+  if (strncmp(mainw->msg,"completed",9)) {
+    if (rfx->num_in_channels>0) {
+      max_frame=atoi(mainw->msg);
+    }
+    else {
+      gint numtok=get_token_count (mainw->msg,'|');
+      if (numtok>4) {
+	gchar **array=g_strsplit(mainw->msg,"|",numtok);
+	max_frame=atoi(array[0]);
+	cfile->hsize=atoi(array[1]);
+	cfile->vsize=atoi(array[2]);
+	// TODO - ** small generated frames may not work with framedrawing - may need to calc offset between frame and image
+	mainw->fd_scale=calc_fd_scale(cfile->hsize,cfile->vsize);
+	cfile->fps=cfile->pb_fps=strtod(array[3],NULL);
+	if (cfile->fps==0) cfile->fps=cfile->pb_fps=prefs->default_fps;
+	tot_frames=atoi(array[4]);
+	g_strfreev(array);
+      }
     }
-    pixbuf=gdk_pixbuf_new_from_file(buf,&error);
-  } while (!(error==NULL));
-
-  if (pixbuf!=NULL&&mainw->fd_scale>1.) {
-    pixbuf2=gdk_pixbuf_scale_simple(pixbuf,(gdouble)cfile->hsize/mainw->fd_scale,(gdouble)cfile->vsize/mainw->fd_scale,GDK_INTERP_HYPER);
-    gdk_pixbuf_unref(pixbuf);
-    pixbuf=pixbuf2;
   }
+
+  if (max_frame>0) {
+    if (rfx->num_in_channels==0) {
+      gtk_spin_button_set_range (GTK_SPIN_BUTTON (mainw->framedraw_spinbutton),1,tot_frames);
+    }
+    
+    if (mainw->framedraw_frame>max_frame) {
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->framedraw_spinbutton),max_frame);
+      mainw->current_file=current_file;
+      return;
+    }
+  }
+
+  if (rfx->num_in_channels>0) {
+    img_ext="pre";
+  }
+  else {
+    img_ext=cfile->img_type==IMG_TYPE_JPEG?"jpg":"png";
+  }
+
+  tc=((mainw->framedraw_frame-1.))/cfile->fps*U_SECL;
+  pixbuf=pull_gdk_pixbuf_at_size(mainw->current_file,mainw->framedraw_frame,(gchar *)img_ext,tc,(gdouble)cfile->hsize/mainw->fd_scale,(gdouble)cfile->vsize/mainw->fd_scale,GDK_INTERP_HYPER);
+
   gtk_image_set_from_pixbuf(GTK_IMAGE(mainw->framedraw_image), pixbuf);
   gtk_widget_queue_draw (mainw->framedraw_image);
+  mainw->current_file=current_file;
+  if (pixbuf!=NULL) gdk_pixbuf_unref(pixbuf);
+
+  lives_set_cursor_style(LIVES_CURSOR_NORMAL,NULL);
 }
 
 
@@ -429,7 +500,7 @@ after_framedraw_frame_spinbutton_changed (GtkSpinButton *spinbutton, lives_speci
 {
   // update the single frame/framedraw preview
   mainw->framedraw_frame=gtk_spin_button_get_value_as_int(spinbutton);
-  if (!(framedraw->rfx->props&RFX_PROPS_MAY_RESIZE)) load_rfx_preview();
+  if (GTK_WIDGET_VISIBLE(mainw->framedraw_preview)) load_rfx_preview(framedraw->rfx);
   else framedraw_redraw(framedraw, TRUE, NULL);
 }
 
@@ -447,27 +518,20 @@ load_framedraw_image(GdkPixbuf *pixbuf) {
 
   // see for example framedraw_redraw for how to combine with a mask
 
-  GError *error=NULL;
-  gchar buf[256];
   GdkBitmap *dummy_bitmap;
-  GdkPixbuf *pixbuf2;
+  weed_timecode_t tc;
+  gboolean needs_free=FALSE;
+
+  if (mainw->framedraw_frame>cfile->frames) mainw->framedraw_frame=cfile->frames;
+
+  // can happen if we preview for rendered generators
+  if (mainw->multitrack==NULL&&mainw->framedraw_frame==0) return;
 
   if (pixbuf==NULL) {
-    // TODO - use load_frame func
-    g_snprintf(buf,256,"%s/%s/%08d.%s",prefs->tmpdir,cfile->handle,mainw->framedraw_frame,prefs->image_ext);
-    do {
-      if (!(error==NULL)) {
-	g_error_free(error);
-	error=NULL;
-      }
-      pixbuf=gdk_pixbuf_new_from_file(buf,&error);
-    } while (!(error==NULL));
-    
-    if (pixbuf!=NULL&&mainw->fd_scale>1.) {
-      pixbuf2=gdk_pixbuf_scale_simple(pixbuf,(gdouble)cfile->hsize/mainw->fd_scale,(gdouble)cfile->vsize/mainw->fd_scale,GDK_INTERP_HYPER);
-      gdk_pixbuf_unref(pixbuf);
-      pixbuf=pixbuf2;
-    }
+    const gchar *img_ext=cfile->img_type==IMG_TYPE_JPEG?"jpg":"png";
+    tc=((mainw->framedraw_frame-1.))/cfile->fps*U_SECL;
+    pixbuf=pull_gdk_pixbuf_at_size(mainw->current_file,mainw->framedraw_frame,img_ext,tc,(gdouble)cfile->hsize/mainw->fd_scale,(gdouble)cfile->vsize/mainw->fd_scale,GDK_INTERP_HYPER);
+    needs_free=TRUE;
   }
 
   if (mainw->framedraw_orig_pixmap!=NULL) {
@@ -475,7 +539,9 @@ load_framedraw_image(GdkPixbuf *pixbuf) {
   }
 
   // we just want the pixmap
+
   gdk_pixbuf_render_pixmap_and_mask (pixbuf, &mainw->framedraw_orig_pixmap, &dummy_bitmap, 128);
+  if (needs_free&&pixbuf!=NULL) gdk_pixbuf_unref(pixbuf);
 
   if (!(dummy_bitmap==NULL)) {
     gdk_pixmap_unref (dummy_bitmap);
@@ -634,14 +700,14 @@ on_framedraw_mouse_update (GtkWidget *widget, GdkEventButton *event, lives_speci
       gdouble scalex;
       gdouble scaley;
       if (mainw->multitrack!=NULL) {
-	// scale is rel. to in channel size (inwidth, inheight)
-	scalex=(gdouble)(xcurrent-xstart)/(gdouble)mainw->multitrack->inwidth;
-	scaley=(gdouble)(ycurrent-ystart)/(gdouble)mainw->multitrack->inheight;
-	if (mainw->multitrack->inwidth<mainw->multitrack->play_window_width) {
-	  scalex=(gdouble)(xcurrent-xstart)/(gdouble)mainw->multitrack->inwidth;
+	// scale is rel. to out channel size (outwidth, outheight)
+	scalex=(gdouble)(xcurrent-xstart)/(gdouble)mainw->multitrack->outwidth;
+	scaley=(gdouble)(ycurrent-ystart)/(gdouble)mainw->multitrack->outheight;
+	if (mainw->multitrack->outwidth<mainw->multitrack->play_window_width) {
+	  scalex=(gdouble)(xcurrent-xstart)/(gdouble)mainw->multitrack->outwidth;
 	}
-	if (mainw->multitrack->inheight<mainw->multitrack->play_window_height) {
-	  scaley=(gdouble)(ycurrent-ystart)/(gdouble)mainw->multitrack->inheight;
+	if (mainw->multitrack->outheight<mainw->multitrack->play_window_height) {
+	  scaley=(gdouble)(ycurrent-ystart)/(gdouble)mainw->multitrack->outheight;
 	}
       }
       else {
@@ -742,7 +808,7 @@ draw_rect_demask (GdkColor *col, gint x1, gint y1, gint x2, gint y2, gboolean fi
 
   gchar *com;
   com=g_strdup_printf ("/usr/X11R6/lib/xscreensaver/goop -window-id %lu 2>/dev/null&",GDK_WINDOW_XWINDOW (mainw->framedraw->window));
-  system (com);
+  dummyvar=system (com);
   g_free (com);
 		       
 #endif
