@@ -1,7 +1,7 @@
 // jack.c
 // LiVES (lives-exe)
-// (c) G. Finch 2005 - 2006
-// Released under the GPL 2.0 or higher
+// (c) G. Finch 2005 - 2009
+// Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
 #include "main.h"
@@ -11,12 +11,9 @@
 #ifdef ENABLE_JACK
 #include "callbacks.h"
 #include "support.h"
-#include "audio.h"
 
 #define afile mainw->files[jackd->playing_file]
 
-extern void weed_apply_audio_instance(weed_plant_t *init_event, float **abuf, int nchans, long nsamps, int fileno, weed_timecode_t tc); // defined in effects-weed.c
-
 static jack_client_t *jack_transport_client;
 
 int lives_start_ready_callback (jack_transport_state_t state, jack_position_t *pos, void *arg) {
@@ -37,6 +34,31 @@ void lives_jack_init (void) {
   if ((prefs->jack_opts&JACK_OPTS_START_TSERVER)||(prefs->jack_opts&JACK_OPTS_START_ASERVER)) {
     unsetenv ("JACK_NO_START_SERVER");
     setenv ("JACK_START_SERVER","1",0);
+
+    if (!g_file_test(prefs->jack_aserver,G_FILE_TEST_EXISTS)) {
+      gchar *com;
+      gchar jackd_loc[512];
+      get_location("jackd",jackd_loc,512);
+      if (strlen(jackd_loc)) {
+#ifndef IS_DARWIN
+	com=g_strdup_printf("echo \"%s -Z -d alsa\">%s",jackd_loc,prefs->jack_aserver);
+#else
+#ifdef IS_SOLARIS
+	// use OSS on Darwin
+	com=g_strdup_printf("echo \"%s -Z -d oss\">%s",jackd_loc,prefs->jack_aserver);
+#else
+	// use coreaudio on Darwin
+	com=g_strdup_printf("echo \"%s -Z -d coreaudio\">%s",jackd_loc,prefs->jack_aserver);
+#endif
+#endif
+	dummyvar=system(com);
+	g_free(com);
+	com=g_strdup_printf("/bin/chmod o+x %s",prefs->jack_aserver);
+	dummyvar=system(com);
+	g_free(com);
+      }
+    }
+
   }
   else {
     unsetenv ("JACK_START_SERVER");
@@ -67,7 +89,7 @@ void lives_jack_init (void) {
 // transport handling
 
 
-void jack_transport_check_state (void) {
+static void jack_transport_check_state (void) {
 #ifdef ENABLE_JACK_TRANSPORT
   jack_position_t pos;
   jack_transport_state_t jacktstate;
@@ -93,7 +115,7 @@ gboolean lives_jack_poll(gpointer data) {
   // data is always NULL
   // must return TRUE
 #ifdef ENABLE_JACK_TRANSPORT
-  if (!mainw->is_processing) jack_transport_check_state();
+  jack_transport_check_state();
 #endif
   return TRUE;
 }
@@ -127,16 +149,43 @@ static jack_driver_t outdev[JACK_MAX_OUTDEVICES];
 static jack_driver_t indev[JACK_MAX_OUTDEVICES];
 
 
+static float set_pulse(float *buf, size_t bufsz, int step) {
+  float *ptr=buf;
+  float *end=buf+bufsz;
+
+  float tot;
+  int count=0;
+
+  while (ptr<end) {
+    tot+=*ptr;
+    count++;
+    ptr+=step;
+  }
+  if (count>0) return tot/(float)count;
+  return 0.;
+}
+
+
+void jack_get_rec_avals(jack_driver_t *jackd) {
+  mainw->rec_aclip=jackd->playing_file;
+  if (mainw->rec_aclip!=-1) {
+    mainw->rec_aseek=jackd->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
+    mainw->rec_avel=afile->pb_fps/afile->fps;
+  }
+}
+
+
 static int audio_process (nframes_t nframes, void *arg) {
   // JACK calls this periodically to get the next audio buffer
   float* out_buffer[JACK_MAX_OUTPUT_PORTS];
   jack_driver_t* jackd = (jack_driver_t*)arg;
   jack_position_t pos;
   register int i;
-  jack_message_t *msg;
+  aserver_message_t *msg;
   long seek,xseek;
   int new_file;
   gchar *filename;
+  gboolean from_memory=FALSE;
 
 #ifdef DEBUG_AJACK
   g_printerr("nframes %ld, sizeof(float) == %d\n", (long)nframes, sizeof(float));
@@ -145,13 +194,13 @@ static int audio_process (nframes_t nframes, void *arg) {
   if (!mainw->is_ready||jackd==NULL||(mainw->playing_file==-1&&jackd->is_silent&&jackd->msgq==NULL)) return 0;
 
   /* process one message */
-  if((msg=(jack_message_t *)jackd->msgq)!=NULL) {
+  if((msg=(aserver_message_t *)jackd->msgq)!=NULL) {
 
     switch (msg->command) {
-    case JACK_CMD_FILE_OPEN:
+    case ASERVER_CMD_FILE_OPEN:
       new_file=atoi(msg->data);
       if (jackd->playing_file!=new_file) {
-	if (jackd->has_header) filename=g_strdup_printf("%s/%s/audiodump.wav",prefs->tmpdir,mainw->files[new_file]->handle);
+	if (jackd->is_opening) filename=g_strdup_printf("%s/%s/audiodump.pcm",prefs->tmpdir,mainw->files[new_file]->handle);
 	else filename=g_strdup_printf("%s/%s/audio",prefs->tmpdir,mainw->files[new_file]->handle);
 	jackd->fd=open(filename,O_RDONLY);
 	if (jackd->fd==-1) {
@@ -162,14 +211,13 @@ static int audio_process (nframes_t nframes, void *arg) {
 	  if (jackd->aPlayPtr->data!=NULL) g_free(jackd->aPlayPtr->data);
 	  if (jackd->buffer_size>0) jackd->aPlayPtr->data=g_malloc(jackd->buffer_size*100);
 	  else (jackd->aPlayPtr->data)=NULL;
-	  if (jackd->has_header) jackd->seek_pos=WAV_HEADER_BYTES;
-	  else jackd->seek_pos=0;
+	  jackd->seek_pos=0;
 	  jackd->playing_file=new_file;
 	}
 	g_free(filename);
       }
       break;
-    case JACK_CMD_FILE_CLOSE:
+    case ASERVER_CMD_FILE_CLOSE:
       if (jackd->fd>0) close(jackd->fd);
       jackd->fd=0;
       if (jackd->aPlayPtr->data!=NULL) g_free(jackd->aPlayPtr->data);
@@ -177,33 +225,32 @@ static int audio_process (nframes_t nframes, void *arg) {
       jackd->aPlayPtr->size=0;
       jackd->playing_file=-1;
       break;
-    case JACK_CMD_FILE_SEEK:
+    case ASERVER_CMD_FILE_SEEK:
       if (jackd->fd<1) break;
       xseek=seek=atol(msg->data);
       if (seek<0.) xseek=0.;
-      if (jackd->has_header) seek+=WAV_HEADER_BYTES;
       if (!jackd->mute) {
 	lseek(jackd->fd,xseek,SEEK_SET);
       }
       jackd->seek_pos=seek;
       gettimeofday(&tv, NULL);
-      jackd->audio_ticks=U_SECL*(tv.tv_sec-mainw->startsecs)+tv.tv_usec*U_SECL/1000000;
+      jackd->audio_ticks=U_SECL*(tv.tv_sec-mainw->startsecs)+tv.tv_usec*U_SEC_RATIO;
       jackd->frames_written=0;
       break;
     default:
       msg->data=NULL;
     }
     if (msg->data!=NULL) g_free(msg->data);
-    msg->command=JACK_CMD_PROCESSED;
+    msg->command=ASERVER_CMD_PROCESSED;
     jackd->msgq = NULL;    /* take this message off of the queue */
 
     return 0;
   }  
 
-  if(nframes != jackd->chunk_size) jackd->chunk_size = nframes;
+  if (nframes != jackd->chunk_size) jackd->chunk_size = nframes;
 
   /* retrieve the buffers for the output ports */
-  for(i = 0; i < jackd->num_output_channels; i++) out_buffer[i] = (float *) jack_port_get_buffer(jackd->output_port[i], nframes);
+  for (i = 0; i < jackd->num_output_channels; i++) out_buffer[i] = (float *) jack_port_get_buffer(jackd->output_port[i], nframes);
 
   jackd->state=jack_transport_query (jackd->client, &pos);
 
@@ -212,7 +259,7 @@ static int audio_process (nframes_t nframes, void *arg) {
 #endif
 
   /* handle playing state */
-  if(jackd->state==JackTransportRolling||jackd->play_when_stopped) {
+  if (jackd->state==JackTransportRolling||jackd->play_when_stopped) {
     gulong jackFramesAvailable = nframes; /* frames we have left to write to jack */
     gulong inputFramesAvailable;          /* frames we have available this loop */
     gulong numFramesToWrite;              /* num frames we are writing this loop */
@@ -222,26 +269,28 @@ static int audio_process (nframes_t nframes, void *arg) {
 
     guchar* buffer;
 
+    gdouble vol;
+
 #ifdef DEBUG_AJACK
     g_printerr("playing... jackFramesAvailable = %ld\n", jackFramesAvailable);
 #endif
 
     jackd->num_calls++;
 
-    if (!jackd->in_use||jackd->fd<1||jackd->is_paused||jackd->seek_pos<0.) {
+    if (!jackd->in_use||((jackd->fd<1||jackd->seek_pos<0.)&&jackd->read_abuf<0)||jackd->is_paused) {
       /* output silence if nothing is being outputted */
       if (!jackd->is_silent) for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
       jackd->is_silent=TRUE;
       if (!jackd->is_paused) jackd->frames_written+=nframes;
       if (jackd->seek_pos<0.&&jackd->playing_file>-1&&afile!=NULL) {
 	jackd->seek_pos+=nframes*afile->achans*afile->asampsize/8;
-	if (jackd->seek_pos>=0) audio_seek_bytes(jackd,jackd->seek_pos);
+	if (jackd->seek_pos>=0) jack_audio_seek_bytes(jackd,jackd->seek_pos);
       }
       return 0;
     }
 
 
-    if(jackd->buffer_size<(jackFramesAvailable*sizeof(short)*jackd->num_output_channels)) {
+    if (jackd->buffer_size<(jackFramesAvailable*sizeof(short)*jackd->num_output_channels)) {
       //ERR("our buffer must have changed size\n");
       //ERR("allocated %ld bytes, need %ld bytes\n", jackd->buffer_size,
       // jackFramesAvailable * sizeof(short) * jackd->num_output_channels);
@@ -250,101 +299,102 @@ static int audio_process (nframes_t nframes, void *arg) {
 
     jackd->is_silent=FALSE;
 
-    if (G_LIKELY(jackFramesAvailable>0&&jackd->aPlayPtr!=NULL&&jackd->aPlayPtr->data!=NULL&&jackd->num_input_channels>0)) {
+    if (G_LIKELY(jackFramesAvailable>0&&(jackd->read_abuf>-1||(jackd->aPlayPtr!=NULL&&jackd->aPlayPtr->data!=NULL&&jackd->num_input_channels>0)))) {
       /* (bytes of data) / (2 bytes(16 bits) * X input channels) == frames */
-      if (G_LIKELY(jackd->fd>0)) {
-	jackd->aPlayPtr->size=0;
-	in_bytes=ABS((in_frames=((gdouble)jackd->sample_in_rate/(gdouble)jackd->sample_out_rate*(gdouble)jackFramesAvailable+(fastrand()/G_MAXUINT32))))*jackd->num_input_channels*jackd->bytes_per_channel;
-	if ((shrink_factor=(gfloat)in_frames/(gfloat)jackFramesAvailable)<0.f) {
-	  // reverse playback
-	  if ((jackd->seek_pos-=in_bytes)<(WAV_HEADER_BYTES*jackd->has_header)) {
-	    if (jackd->loop==JACK_LOOP_NONE) {
+
+      if (mainw->playing_file>-1&&jackd->read_abuf>-1) {
+	// playing back from memory buffers instead of from file
+	// this is used in multitrack
+	from_memory=TRUE;
+
+	numFramesToWrite=jackFramesAvailable;
+	jackd->frames_written+=numFramesToWrite;
+	jackFramesAvailable-=numFramesToWrite; /* take away what was written */
+
+      }
+      else {
+	if (G_LIKELY(jackd->fd>0)) {
+	  jackd->aPlayPtr->size=0;
+	  in_bytes=ABS((in_frames=((gdouble)jackd->sample_in_rate/(gdouble)jackd->sample_out_rate*(gdouble)jackFramesAvailable+(fastrand()/G_MAXUINT32))))*jackd->num_input_channels*jackd->bytes_per_channel;
+	  if ((shrink_factor=(gfloat)in_frames/(gfloat)jackFramesAvailable)<0.f) {
+	    // reverse playback
+	    if ((jackd->seek_pos-=in_bytes)<0) {
+	      if (jackd->loop==AUDIO_LOOP_NONE) {
+		if (*jackd->whentostop==STOP_ON_AUD_END) {
+		  *jackd->cancelled=CANCEL_AUD_END;
+		}
+		jackd->in_use=FALSE;
+		mainw->rec_aclip=jackd->playing_file;
+		mainw->rec_avel=-afile->pb_fps/afile->fps;
+		mainw->rec_aseek=(gdouble)jackd->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
+	      }
+	      else {
+		if (jackd->loop==AUDIO_LOOP_PINGPONG) {
+		  jackd->sample_in_rate=-jackd->sample_in_rate;
+		  shrink_factor=-shrink_factor;
+		  jackd->seek_pos=0;
+		}
+		else jackd->seek_pos=jackd->seek_end-in_bytes;
+		mainw->rec_aclip=jackd->playing_file;
+		mainw->rec_avel=-afile->pb_fps/afile->fps;
+		mainw->rec_aseek=(gdouble)jackd->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
+	      }
+	    }
+	    // rewind by in_bytes
+	    lseek(jackd->fd,jackd->seek_pos,SEEK_SET);
+	  }
+	  if (jackd->mute) {
+	    if (shrink_factor>0.f) jackd->seek_pos+=in_bytes;
+	    if (jackd->seek_pos>=jackd->seek_end) {
 	      if (*jackd->whentostop==STOP_ON_AUD_END) {
 		*jackd->cancelled=CANCEL_AUD_END;
 	      }
-	      jackd->in_use=FALSE;
-	      if (mainw->record&&prefs->rec_opts&REC_AUDIO) {
-		mainw->rec_aclip=jackd->playing_file;
-		mainw->rec_avel=-afile->pb_fps/afile->fps;
-		mainw->rec_aseek=(gdouble)jackd->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
-	      }
-	    }
-	    else {
-	      if (jackd->loop==JACK_LOOP_PINGPONG) {
-		jackd->sample_in_rate=-jackd->sample_in_rate;
-		shrink_factor=-shrink_factor;
-		jackd->seek_pos=0+(WAV_HEADER_BYTES*jackd->has_header);
-	      }
-	      else jackd->seek_pos=jackd->seek_end-in_bytes;
-	      if (mainw->record&&prefs->rec_opts&REC_AUDIO) {
-		mainw->rec_aclip=jackd->playing_file;
-		mainw->rec_avel=-afile->pb_fps/afile->fps;
-		mainw->rec_aseek=(gdouble)jackd->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
-	      }
-	    }
-	  }
-	  // rewind by in_bytes
-	  lseek(jackd->fd,jackd->seek_pos,SEEK_SET);
-	}
-	if (jackd->mute) {
-	  if (shrink_factor>0.f) jackd->seek_pos+=in_bytes;
-	  if (jackd->seek_pos>=jackd->seek_end) {
-	    if (*jackd->whentostop==STOP_ON_AUD_END) {
-	      *jackd->cancelled=CANCEL_AUD_END;
-	    }
-	    else {
-	      if (jackd->loop==JACK_LOOP_PINGPONG) {
-		jackd->sample_in_rate=-jackd->sample_in_rate;
-		jackd->seek_pos-=in_bytes;
-	      }
 	      else {
-		seek=0;
-		if (jackd->has_header) seek+=WAV_HEADER_BYTES;
-		jackd->seek_pos=seek;
+		if (jackd->loop==AUDIO_LOOP_PINGPONG) {
+		  jackd->sample_in_rate=-jackd->sample_in_rate;
+		  jackd->seek_pos-=in_bytes;
+		}
+		else {
+		  seek=0;
+		  jackd->seek_pos=seek;
+		}
 	      }
 	    }
+	    for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
+	    jackd->frames_written+=nframes;
+	    return 0;
 	  }
-	  for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
-	  jackd->frames_written+=nframes;
-	  return 0;
-	}
-	else {
-	  gboolean loop_restart;
-	  do {
-	    loop_restart=FALSE;
-	    if (in_bytes>0) {
-	      // playing from a file
-	      if (!(*jackd->cancelled)&&ABS(shrink_factor)<=100.f) {
-		if ((jackd->aPlayPtr->size=read(jackd->fd,jackd->aPlayPtr->data,in_bytes))==0) {
-		  if (*jackd->whentostop==STOP_ON_AUD_END) {
-		    *jackd->cancelled=CANCEL_AUD_END;
-		  }
-		  else {
-		    loop_restart=TRUE;
-		    if (jackd->loop==JACK_LOOP_PINGPONG) {
-		      jackd->sample_in_rate=-jackd->sample_in_rate;
-		      lseek(jackd->fd,(jackd->seek_pos-=in_bytes),SEEK_SET);
-		      if (mainw->record&&prefs->rec_opts&REC_AUDIO) {
+	  else {
+	    gboolean loop_restart;
+	    do {
+	      loop_restart=FALSE;
+	      if (in_bytes>0) {
+		// playing from a file
+		if (!(*jackd->cancelled)&&ABS(shrink_factor)<=100.f) {
+		  if ((jackd->aPlayPtr->size=read(jackd->fd,jackd->aPlayPtr->data,in_bytes))==0) {
+		    if (*jackd->whentostop==STOP_ON_AUD_END) {
+		      *jackd->cancelled=CANCEL_AUD_END;
+		    }
+		    else {
+		      loop_restart=TRUE;
+		      if (jackd->loop==AUDIO_LOOP_PINGPONG) {
+			jackd->sample_in_rate=-jackd->sample_in_rate;
+			lseek(jackd->fd,(jackd->seek_pos-=in_bytes),SEEK_SET);
 			mainw->rec_aclip=jackd->playing_file;
 			mainw->rec_avel=-afile->pb_fps/afile->fps;
 			mainw->rec_aseek=(gdouble)jackd->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
 		      }
-		    }
-		    else {
-		      if (jackd->loop!=JACK_LOOP_NONE) {
-			seek=0;
-			if (jackd->has_header) seek+=WAV_HEADER_BYTES;
-			lseek(jackd->fd,(jackd->seek_pos=seek),SEEK_SET);
-			if (mainw->record&&prefs->rec_opts&REC_AUDIO) {
-			  mainw->rec_aclip=jackd->playing_file;
-			  mainw->rec_avel=-afile->pb_fps/afile->fps;
-			  mainw->rec_aseek=(gdouble)jackd->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
-			}
-		      }
 		      else {
-			jackd->in_use=FALSE;
-			loop_restart=FALSE;
-			if (mainw->record&&prefs->rec_opts&REC_AUDIO) {
+			if (jackd->loop!=AUDIO_LOOP_NONE) {
+			  seek=0;
+			  lseek(jackd->fd,(jackd->seek_pos=seek),SEEK_SET);
+			  mainw->rec_aclip=jackd->playing_file;
+			  mainw->rec_avel=-afile->pb_fps/afile->fps;
+			  mainw->rec_aseek=(gdouble)jackd->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
+			}
+			else {
+			  jackd->in_use=FALSE;
+			  loop_restart=FALSE;
 			  mainw->rec_aclip=jackd->playing_file;
 			  mainw->rec_avel=-afile->pb_fps/afile->fps;
 			  mainw->rec_aseek=(gdouble)jackd->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
@@ -352,74 +402,94 @@ static int audio_process (nframes_t nframes, void *arg) {
 		      }
 		    }
 		  }
-		}
-		else {
-		  if (shrink_factor<0.f) {
-		    // reverse play - rewind again by in_bytes
-		    lseek(jackd->fd,jackd->seek_pos,SEEK_SET);
+		  else {
+		    if (shrink_factor<0.f) {
+		      // reverse play - rewind again by in_bytes
+		      lseek(jackd->fd,jackd->seek_pos,SEEK_SET);
+		    }
+		    else jackd->seek_pos+=jackd->aPlayPtr->size;
 		  }
-		  else jackd->seek_pos+=jackd->aPlayPtr->size;
 		}
 	      }
-	    }
-	  } while (loop_restart);
+	    } while (loop_restart);
+	  }
+	}
+
+
+	if (!jackd->in_use||in_bytes==0) {
+	  // reached end of audio with no looping
+	  for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
+	  jackd->is_silent=TRUE;
+	  if (!jackd->is_paused) jackd->frames_written+=nframes;
+	  if (jackd->seek_pos<0.&&jackd->playing_file>-1&&afile!=NULL) {
+	    jackd->seek_pos+=nframes*afile->achans*afile->asampsize/8;
+	    if (jackd->seek_pos>=0) jack_audio_seek_bytes(jackd,jackd->seek_pos);
+	  }
+	  return 0;
+	}
+	
+	inputFramesAvailable = jackd->aPlayPtr->size / (jackd->num_input_channels * jackd->bytes_per_channel);
+#ifdef DEBUG_AJACK
+	g_printerr("%d inputFramesAvailable == %ld, %ld, %ld %ld,jackFramesAvailable == %ld\n", jackd->aPlayPtr->size, inputFramesAvailable, in_frames,jackd->sample_in_rate,jackd->sample_out_rate,jackFramesAvailable);
+#endif
+	buffer = jackd->aPlayPtr->data;
+	
+	numFramesToWrite = MIN(jackFramesAvailable, inputFramesAvailable/ABS(shrink_factor)); /* write as many bytes as we have space remaining, or as much as we have data to write */
+	
+#ifdef DEBUG_AJACK
+	g_printerr("inputFramesAvailable after conversion %ld\n", (gulong)((gdouble)inputFramesAvailable/shrink_factor));
+	g_printerr("nframes == %d, jackFramesAvailable == %ld,\n\tjackd->num_input_channels == %ld, jackd->num_output_channels == %ld\n",  nframes, jackFramesAvailable, jackd->num_input_channels, jackd->num_output_channels);
+#endif
+	
+	/* convert from 8 bit to 16 bit and mono to stereo if necessary */
+	/* resample as we go */
+	if(jackd->bytes_per_channel==1) {
+	  sample_move_d8_d16 ((short *)(jackd->sound_buffer + (nframes-jackFramesAvailable) / sizeof(short)),(guchar *)buffer, numFramesToWrite, in_bytes, shrink_factor, jackd->num_output_channels, jackd->num_input_channels,0);
+	}
+	/* 16 bit input samples */
+	/* resample as we go */
+	else {
+	  sample_move_d16_d16((short*)jackd->sound_buffer + ((nframes - jackFramesAvailable) * jackd->bytes_per_channel * jackd->num_output_channels) / sizeof(short), (short*)buffer, numFramesToWrite, in_bytes, shrink_factor, jackd->num_output_channels, jackd->num_input_channels, jackd->reverse_endian?SWAP_X_TO_L:0, 0);
+	}
+	
+	jackd->frames_written+=numFramesToWrite;
+	jackFramesAvailable-=numFramesToWrite; /* take away what was written */
+	
+#ifdef DEBUG_AJACK
+	g_printerr("jackFramesAvailable == %ld\n", jackFramesAvailable);
+#endif
+      }
+      
+      // playback from memory or file
+      
+      
+      vol=mainw->volume*mainw->volume; // TODO - we should really use an exponential scale
+      
+      if (!from_memory) {
+
+	if (((gint)(jackd->num_calls/100.))*100==jackd->num_calls) if (mainw->soft_debug) g_print("audio pip\n");
+	
+	for (i=0;i<jackd->num_output_channels;i++) {
+	  sample_move_d16_float(out_buffer[i], (short*)jackd->sound_buffer + i, (nframes - jackFramesAvailable), jackd->num_output_channels, afile->signed_endian&AFORM_UNSIGNED, vol);
 	}
       }
-
-      if (!jackd->in_use||in_bytes==0) {
-	// reached end of audio with no looping
-	for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
-	jackd->is_silent=TRUE;
-	if (!jackd->is_paused) jackd->frames_written+=nframes;
-	if (jackd->seek_pos<0.&&jackd->playing_file>-1&&afile!=NULL) {
-	  jackd->seek_pos+=nframes*afile->achans*afile->asampsize/8;
-	  if (jackd->seek_pos>=0) audio_seek_bytes(jackd,jackd->seek_pos);
-	}
-	return 0;
-      }
-
-      inputFramesAvailable = jackd->aPlayPtr->size / (jackd->num_input_channels * jackd->bytes_per_channel);
-#ifdef DEBUG_AJACK
-      g_printerr("%d inputFramesAvailable == %ld, %ld, %ld %ld,jackFramesAvailable == %ld\n", jackd->aPlayPtr->size, inputFramesAvailable, in_frames,jackd->sample_in_rate,jackd->sample_out_rate,jackFramesAvailable);
-#endif
-      buffer = jackd->aPlayPtr->data;
-
-      numFramesToWrite = MIN(jackFramesAvailable, inputFramesAvailable/ABS(shrink_factor)); /* write as many bytes as we have space remaining, or as much as we have data to write */
-
-#ifdef DEBUG_AJACK
-      g_printerr("inputFramesAvailable after conversion %ld\n", (gulong)((gdouble)inputFramesAvailable/shrink_factor));
-      g_printerr("nframes == %d, jackFramesAvailable == %ld,\n\tjackd->num_input_channels == %ld, jackd->num_output_channels == %ld\n",  nframes, jackFramesAvailable, jackd->num_input_channels, jackd->num_output_channels);
-#endif
-
-      /* convert from 8 bit to 16 bit and mono to stereo if necessary */
-      /* resample as we go */
-      if(jackd->bytes_per_channel==1) {
-	sample_move_d8_d16 ((short *)(jackd->sound_buffer + (nframes-jackFramesAvailable) / sizeof(short)),
-			    (guchar *)buffer, numFramesToWrite, shrink_factor, jackd->num_output_channels, jackd->num_input_channels);
-      }
-      /* 16 bit input samples */
-      /* resample as we go */
       else {
-	sample_move_d16_d16((short*)jackd->sound_buffer + ((nframes - jackFramesAvailable) * jackd->bytes_per_channel * jackd->num_output_channels) / sizeof(short), (short*)buffer, numFramesToWrite, shrink_factor, jackd->num_output_channels, jackd->num_input_channels, jackd->reverse_endian);
+	if (jackd->read_abuf>-1&&!jackd->mute) {
+	  sample_move_abuf_float(out_buffer,jackd->num_output_channels,nframes,jackd->sample_out_rate,vol);
+	}
+	else {
+	  for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
+	}
       }
-
-      jackd->frames_written+=numFramesToWrite;
-      jackFramesAvailable-=numFramesToWrite; /* take away what was written */
-
-#ifdef DEBUG_AJACK
-      g_printerr("jackFramesAvailable == %ld\n", jackFramesAvailable);
-#endif
-    }
-
-    for(i=0;i<jackd->num_output_channels;i++) {
-      sample_move_d16_float(out_buffer[i], (short*)jackd->sound_buffer + i, (nframes - jackFramesAvailable), jackd->num_output_channels, afile->signed_endian&AFORM_UNSIGNED, mainw->volume);
-    }
-
-    if (mainw->multitrack!=NULL&&mainw->multitrack->avol_init_event!=NULL) {
-      weed_timecode_t tc=(weed_timecode_t)jack_get_time(jackd);
-      weed_apply_audio_instance(mainw->multitrack->avol_init_event,out_buffer,jackd->num_output_channels,(nframes-jackFramesAvailable),jackd->playing_file,tc);
     }
 
+    /*    jackd->jack_pulse[0]=set_pulse(out_buffer[0],jack->buffer_size,8);
+	  if (jackd->num_output_channels>1) {
+	  jackd->jack_pulse[1]=set_pulse(out_buffer[1],jackd->buffer_size,8);
+	  }
+	  else jackd->jack_pulse[1]=jackd->jack_pulse[0];
+    */
+    
     if(jackFramesAvailable) {
 #ifdef DEBUG_AJACK
       g_printerr("buffer underrun of %ld frames\n", jackFramesAvailable);
@@ -465,7 +535,9 @@ static int audio_read (nframes_t nframes, void *arg) {
 
   if (mainw->effects_paused) return 0; // pause during record
 
-  holding_buff=malloc(nframes*afile->achans*afile->asampsize/8);
+  frames_out=(long)((gdouble)nframes/out_scale+.5);
+
+  holding_buff=malloc(frames_out*afile->achans*afile->asampsize/8);
 
   if (nframes != jackd->chunk_size) jackd->chunk_size = nframes;
 
@@ -473,7 +545,7 @@ static int audio_read (nframes_t nframes, void *arg) {
     in_buffer[i] = (float *) jack_port_get_buffer(jackd->input_port[i], nframes);
   }
 
-  frames_out=sample_move_float_int((void *)holding_buff,in_buffer,nframes,out_scale,afile->achans,afile->asampsize,out_unsigned,jackd->reverse_endian);
+  frames_out=sample_move_float_int((void *)holding_buff,in_buffer,nframes,out_scale,afile->achans,afile->asampsize,out_unsigned,jackd->reverse_endian,1.);
 
   jackd->frames_written+=nframes;
 
@@ -482,7 +554,7 @@ static int audio_read (nframes_t nframes, void *arg) {
     mainw->rec_samples-=frames_out;
   }
 
-  write (mainw->aud_rec_fd,holding_buff,frames_out*(afile->asampsize/8)*afile->achans);
+  dummyvar=write (mainw->aud_rec_fd,holding_buff,frames_out*(afile->asampsize/8)*afile->achans);
 
   free(holding_buff);
 
@@ -544,7 +616,7 @@ void jack_shutdown(void* arg) {
   mainw->jackd=jack_get_driver(0,TRUE);
   mainw->jackd->msgq=NULL;
 
-  if (mainw->jackd->playing_file!=-1&&afile!=NULL) audio_seek_bytes(mainw->jackd,mainw->jackd->seek_pos); // at least re-seek to the right place
+  if (mainw->jackd->playing_file!=-1&&afile!=NULL) jack_audio_seek_bytes(mainw->jackd,mainw->jackd->seek_pos); // at least re-seek to the right place
 }
 
 /* Return the difference between two timeval structures in terms of milliseconds */
@@ -599,6 +671,9 @@ static void jack_error_func(const char *desc) {
 }
 
 
+// wait 5 seconds to startup
+#define JACK_START_WAIT 5000000
+
 
 // create a new client and connect it to jack, connect the ports
 int jack_open_device(jack_driver_t *jackd) {
@@ -607,6 +682,9 @@ int jack_open_device(jack_driver_t *jackd) {
   jack_options_t options=JackServerName|JackNoStartServer;
   jack_status_t status;
   int i;
+  
+  struct timeval otv;
+  int64_t ntime=0,stime;
 
   /* zero out the buffer pointer and the size of the buffer */
   jackd->sound_buffer=NULL;
@@ -615,14 +693,22 @@ int jack_open_device(jack_driver_t *jackd) {
   /* set up an error handler */
   jack_set_error_function(jack_error_func);
   jackd->client=NULL;
-  if (!(prefs->jack_opts&JACK_OPTS_START_TSERVER)&&!(prefs->jack_opts&JACK_OPTS_START_ASERVER)) {
-    g_printerr(_("Manual start of jackd required. Please make sure jackd is running, \nor else change the value of <jack_opts> in ~/.lives to 16\nand restart LiVES.\n"));
-  }
-  while (jackd->client==NULL) 
+
+  gettimeofday(&otv, NULL);
+  stime=otv.tv_sec*1000000000+otv.tv_usec;
+
+  while (jackd->client==NULL&&ntime<JACK_START_WAIT) {
     jackd->client = jack_client_open (client_name, options, &status, server_name);
     
+    g_usleep(prefs->sleep_time);
+
+    gettimeofday(&otv, NULL);
+    ntime=(otv.tv_sec*1000000000+otv.tv_usec-stime)>>10;
+  }
+
+
   if (jackd->client==NULL) {
-    g_printerr (g_strdup_printf("jack_client_open() failed, status = 0x%2.0x\n", status));
+    g_printerr ("jack_client_open() failed, status = 0x%2.0x\n", status);
     if (status&JackServerFailed) {
       d_print (_("Unable to connect to JACK server\n"));
     }
@@ -631,7 +717,7 @@ int jack_open_device(jack_driver_t *jackd) {
   
   if (status&JackNameNotUnique) {
     client_name= jack_get_client_name(jackd->client);
-    g_printerr (g_strdup_printf("unique name `%s' assigned\n", client_name));
+    g_printerr ("unique name `%s' assigned\n", client_name);
   }
   
   jackd->sample_out_rate=jack_get_sample_rate(jackd->client);
@@ -703,7 +789,7 @@ int jack_open_device_read(jack_driver_t *jackd) {
     jackd->client = jack_client_open (client_name, options, &status, server_name);
     
   if (jackd->client==NULL) {
-    g_printerr (g_strdup_printf("jack_client_open() failed, status = 0x%2.0x\n", status));
+    g_printerr ("jack_client_open() failed, status = 0x%2.0x\n", status);
     if (status&JackServerFailed) {
       d_print (_("Unable to connect to JACK server\n"));
     }
@@ -712,7 +798,7 @@ int jack_open_device_read(jack_driver_t *jackd) {
   
   if (status&JackNameNotUnique) {
     client_name= jack_get_client_name(jackd->client);
-    g_printerr (g_strdup_printf("unique name `%s' assigned\n", client_name));
+    g_printerr ("unique name `%s' assigned\n", client_name);
   }
   
   jackd->sample_in_rate=jack_get_sample_rate(jackd->client);
@@ -765,6 +851,8 @@ int jack_open_device_read(jack_driver_t *jackd) {
 int jack_driver_activate (jack_driver_t *jackd) {
   // activate client and connect it
 
+  // TODO *** - handle errors here !
+
   int i;
   const char** ports;
   gboolean failed=FALSE;
@@ -789,6 +877,11 @@ int jack_driver_activate (jack_driver_t *jackd) {
       ports=jack_get_ports(jackd->client, jackd->jack_port_name[0], NULL, jackd->jack_port_flags);
     }
 
+    if (ports==NULL) {
+      g_printerr("No jack ports available !\n");
+      return 1;
+    }
+
     /* display a trace of the output ports we found */
 #ifdef DEBUG_JACK_PORTS
     for (i=0;ports[i];i++) g_printerr("ports[%d] = '%s'\n",i,ports[i]);
@@ -1007,6 +1100,7 @@ static void jack_reset_dev(gint dev_idx, gboolean is_output) {
 
 
 int jack_audio_init(void) {
+  // initialise variables
   int i,j;
   jack_driver_t *jackd;
 
@@ -1033,6 +1127,8 @@ int jack_audio_init(void) {
     jackd->mute=FALSE;
     jackd->out_chans_available=0;
     jackd->is_output=TRUE;
+    jackd->read_abuf=-1;
+    jackd->playing_file=-1;
   }
   return 0;
 }
@@ -1069,7 +1165,7 @@ int jack_audio_read_init(void) {
 }
 
 
-volatile jack_message_t *jack_get_msgq(jack_driver_t *jackd) {
+volatile aserver_message_t *jack_get_msgq(jack_driver_t *jackd) {
   // force update - "volatile" doesn't seem to work...
   gchar *tmp=g_strdup_printf("%p %d",jackd->msgq,jackd->jackd_died);
   g_free(tmp);
@@ -1077,48 +1173,52 @@ volatile jack_message_t *jack_get_msgq(jack_driver_t *jackd) {
   return jackd->msgq;
 }
 
-gint64 jack_get_time(jack_driver_t *jackd) {
-  volatile jack_message_t *msg=jackd->msgq;
-  if (msg!=NULL&&msg->command==JACK_CMD_FILE_SEEK) while (jack_get_msgq(jackd)!=NULL); // wait for seek
-  if (jackd->is_output) return jackd->audio_ticks+(gint64)(((gdouble)jackd->frames_written/(gdouble)jackd->sample_out_rate)*U_SEC);
-  return jackd->audio_ticks+(gint64)(((gdouble)jackd->frames_written/(gdouble)jackd->sample_in_rate)*U_SEC);
+gint64 lives_jack_get_time(jack_driver_t *jackd) {
+  volatile aserver_message_t *msg=jackd->msgq;
+  gdouble frames_written=jackd->frames_written;
+  if (frames_written<0.) frames_written=0.;
+  if (msg!=NULL&&msg->command==ASERVER_CMD_FILE_SEEK) while (jack_get_msgq(jackd)!=NULL); // wait for seek
+  if (jackd->is_output) return jackd->audio_ticks+(gint64)(frames_written/(gdouble)jackd->sample_out_rate*U_SEC);
+  return jackd->audio_ticks+(gint64)(frames_written/(gdouble)jackd->sample_in_rate*U_SEC);
 }
 
 
-static jack_message_t jack_message;
+static aserver_message_t jack_message;
 
-void audio_seek_frame (jack_driver_t *jackd, gint frame) {
+void jack_audio_seek_frame (jack_driver_t *jackd, gint frame) {
   // seek to frame "frame" in current file
-  volatile jack_message_t *jmsg;
+  volatile aserver_message_t *jmsg;
   if (frame<1) frame=1;
   long seekstart;
   do {
     jmsg=jack_get_msgq(jackd);
-  } while ((jmsg!=NULL)&&jmsg->command!=JACK_CMD_FILE_SEEK);
+  } while ((jmsg!=NULL)&&jmsg->command!=ASERVER_CMD_FILE_SEEK);
   if (jackd->playing_file==-1) return;
   if (frame>afile->frames) frame=afile->frames;
-  seekstart=(long)((gdouble)(frame-1.)/afile->fps*afile->arate)*afile->achans*afile->asampsize/8;
-  afile->aseek_pos=audio_seek_bytes(jackd,seekstart);
+  seekstart=(long)((gdouble)(frame-1.)/afile->fps*afile->arate)*afile->achans*(afile->asampsize/8);
+  afile->aseek_pos=jack_audio_seek_bytes(jackd,seekstart);
 }
 
 
-long audio_seek_bytes (jack_driver_t *jackd, long bytes) {
+long jack_audio_seek_bytes (jack_driver_t *jackd, long bytes) {
   // seek to relative "secs" in current file
-  volatile jack_message_t *jmsg;
+  volatile aserver_message_t *jmsg;
   long seekstart;
   do {
     jmsg=jack_get_msgq(jackd);
-  } while ((jmsg!=NULL)&&jmsg->command!=JACK_CMD_FILE_SEEK);
+  } while ((jmsg!=NULL)&&jmsg->command!=ASERVER_CMD_FILE_SEEK);
   if (jackd->playing_file==-1) return 0;
   seekstart=((long)(bytes/afile->achans/(afile->asampsize/8)))*afile->achans*(afile->asampsize/8);
-  if (jackd->has_header) seekstart+=WAV_HEADER_BYTES;
+
+  if (seekstart<0) seekstart=0;
   if (seekstart>afile->afilesize) seekstart=afile->afilesize;
-  jack_message.command=JACK_CMD_FILE_SEEK;
+  jack_message.command=ASERVER_CMD_FILE_SEEK;
   jack_message.next=NULL;
   jack_message.data=g_strdup_printf("%ld",seekstart);
   jackd->msgq=&jack_message;
   return seekstart;
 }
 
+#undef afile
 
 #endif
