@@ -1,7 +1,7 @@
 // ldvgrab.h
 // LiVES
 // (c) G. Finch 2006 <salsaman@xs4all.nl>
-// released under the GNU GPL 2.0 or higher
+// released under the GNU GPL 3 or later
 // see file ../COPYING for licensing details
 
 
@@ -41,6 +41,8 @@ struct _dvgrabw {
   GtkWidget *grab;
   GtkWidget *play;
   GtkWidget *quit;
+  GtkWidget *status_entry;
+  GdkCursor *cursor;
   gboolean playing;
   gchar *dirname;
   gchar *filename;
