@@ -1,11 +1,11 @@
 // main.h
 // LiVES
-// (c) G. Finch (salsaman@xs4all.nl) 2003 - 2005
+// (c) G. Finch (salsaman@xs4all.nl) 2003 - 2009
 // see file ../COPYING for full licensing details
 
 /*  This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License version 2 as published by
-    the Free Software Foundation.
+    it under the terms of the GNU General Public License version 3 or higher as 
+    published by the Free Software Foundation.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -82,10 +82,31 @@ POSSIBILITY OF SUCH DAMAGES.
 #define DOC_DIR "/share/doc/lives-"
 
 #define THEME_DIR "/share/lives/themes/"
-#define PLUGIN_DIR "/share/lives/plugins/"
+#define PLUGIN_SCRIPTS_DIR "/share/lives/plugins/"
+#define PLUGIN_EXEC_DIR "/lives/plugins/"
 #define ICON_DIR "/share/lives/icons/"
 #define DATA_DIR "/share/lives/"
-#define LIVES_CONFIG_DIR ".lives-dir"
+#define LIVES_CONFIG_DIR ".lives-dir/"
+
+#define LIVES_DEVICE_DIR "/dev/"
+
+
+#define LIVES_MANUAL_URL "http://lives.sourceforge.net/manual/"
+#define LIVES_MANUAL_FILENAME "LiVES_manual.html"
+#define LIVES_AUTHOR_EMAIL "mailto:salsaman@xs4all.nl"
+#define LIVES_DONATE_URL "https://sourceforge.net/donate/index.php?group_id=64341"
+#define LIVES_BUG_URL "http://sourceforge.net/tracker/?group_id=64341&atid=507139"
+#define LIVES_FEATURE_URL "http://sourceforge.net/tracker/?group_id=64341&atid=507142"
+#define LIVES_TRANSLATE_URL "https://translations.launchpad.net/lives/trunk"
+
+#ifdef IS_DARWIN
+#ifndef off64_t
+#define off64_t off_t
+#endif
+#ifndef lseek64
+#define lseek64 lseek
+#endif
+#endif
 
 // insert methods
 #define AFTER_END 1
@@ -111,10 +132,13 @@ POSSIBILITY OF SUCH DAMAGES.
 #include <liboil/liboil.h>
 #endif
 
+#ifndef IS_SOLARIS
+#define LIVES_INLINE inline
+#endif
 
-// events - we have a relative time and a value; each event type currently has its own array of these
-// reltime is only used to quantise events; after that it is discarded...heh..
 
+// this struct is used only when physically resampling frames on the disk
+// we create an array of these and write them to the disk
 typedef struct {
   gint value;
   gint64 reltime;
@@ -130,26 +154,71 @@ typedef struct {
   GtkWidget *pause_button;
   GtkWidget *preview_button;
   GtkWidget *cancel_button;
+  GtkWidget *scrolledwindow;
   guint frames_done;
 } process;
 
 
+
+
+typedef struct {
+  int afile;
+  double seek;
+  double vel;
+} lives_audio_track_state_t;
+
+
+#include "../libweed/weed.h"
+#include "../libweed/weed-events.h"
+
+// see weed event spec. for more info
+
+
+typedef union {
+  float **floatbuf; // float data - for jack
+  short *int16buf; // 16 bit int - for pulse audio
+} adata;
+
+
+typedef struct {
+
+  int achans;
+  int arate;
+
+  weed_timecode_t start_tc;
+
+  volatile size_t start_sample;  // current read posn
+
+  adata data;
+
+  volatile size_t samples_filled; // number of samples filled (usable)
+  size_t sample_space; // total space
+} lives_audio_buf_t;
+
+
+
+
 // need this for event_list_t *
 #include "events.h"
 
+
+
 typedef struct {                // corresponds to one clip in the GUI
   // cfile - one clip
   //
   // basic info (saved during backup)
   gint bpp;
   gdouble fps;
-  gint hsize;
+  gint hsize; // in pixels (NOT macropixels !)
   gint vsize;
   gint arps; // audio sample rate
   guint signed_endian; // bitfield
 
+  #define AFORM_SIGNED 0
+  #define AFORM_LITTLE_ENDIAN 0
+
   #define AFORM_UNSIGNED 1
-  #define AFORM_BIG_ENDIAN 2
+  #define AFORM_BIG_ENDIAN (1<<1)
   #define AFORM_UNKNOWN 65536
 
   gint arate; // audio playback rate
@@ -181,9 +250,10 @@ typedef struct {                // corresponds to one clip in the GUI
   glong f_size;
   glong afilesize;
   gint old_frames; // for deordering, etc.
-  gchar file_name[256];
+  gchar file_name[256]; // input file
   gchar info_file[256];
   gchar name[256];  // the display name
+  gchar save_file_name[256];
   gchar type[40];
   gint start;
   gint end;
@@ -194,7 +264,7 @@ typedef struct {                // corresponds to one clip in the GUI
   gboolean changed;
   GtkWidget *menuentry;
   gboolean orig_file_name;
-  gboolean was_renamed;;
+  gboolean was_renamed;
   gboolean is_untitled;
   gdouble pb_fps;
   gdouble freeze_fps;
@@ -233,9 +303,6 @@ typedef struct {                // corresponds to one clip in the GUI
 
   // events 
   event *events[1];  //for block resampler
-  gshort event_pending;
-# define EVENT_NONE -1
-# define EVENT_FPS 1
 
   weed_plant_t *event_list;
   weed_plant_t *event_list_back;
@@ -255,10 +322,7 @@ typedef struct {                // corresponds to one clip in the GUI
 #define UNDO_RESIZABLE 2
 #define UNDO_MERGE 3
 
-  // reorder and resample
-#define UNDO_REORDER 4
 
-  // just a resample
 #define UNDO_RESAMPLE 5
 #define UNDO_TRIM_AUDIO 6
 #define UNDO_CHANGE_SPEED 7
@@ -282,6 +346,8 @@ typedef struct {                // corresponds to one clip in the GUI
   // record audio to selection
 #define UNDO_REC_AUDIO 19
 
+#define UNDO_INSERT_WITH_AUDIO 20
+
   gint undo_start;
   gint undo_end;
   gchar undo_text[32];
@@ -308,20 +374,49 @@ typedef struct {                // corresponds to one clip in the GUI
   gint undo_arps; // audio sample rate
 
   gshort clip_type;
-#define CLIP_TYPE_DISK 0
+#define CLIP_TYPE_DISK 0 // imported video, broken into frames
 #define CLIP_TYPE_YUV4MPEG 1
 #define CLIP_TYPE_GENERATOR 2
+#define CLIP_TYPE_FILE 3 // unimported video, not or partially broken in frames
+#define CLIP_TYPE_LIVES2LIVES 4 // type for LiVES to LiVES streaming
   void *ext_src; // points to opaque source for non-disk types
 
+  int *frame_index; // index of frames for CLIP_TYPE_FILE
+  // >0 means corresponding frame within original clip
+  // -1 means corresponding image file (equivalent to CLIP_TYPE_DISK)
+  // size must be >= frames, MUST be contiguous in memory
+
+  int *frame_index_back; // for undo
+  //int nbfi; // number of backup frame indices
+
+  gint fx_frame_pump; // rfx frame pump for virtual clips (CLIP_TYPE_FILE)
+
+#define FX_FRAME_PUMP_VAL 100 // how many frames at a time to pump to rfx
+
+  //int **forward_frame_index; // for rendo
+  //int nffi; // number of forward frame indices
+
+
   gboolean ratio_fps; // if the fps was set by a ratio
-  gboolean variable_fps; // has unquantised frame events (variable fps)
 
   glong aseek_pos; // audio seek posn. (bytes) for when we switch clips
 
+  // decoder data
+
+  gchar mime_type[256];
+
+
+  gboolean deinterlace;
+
+
+#define IMG_TYPE_JPEG (1<<1)
+#define IMG_TYPE_PNG (1<<2)
+
+  guint img_type;
+
   // TODO - change to lives_clip_t
 } file;
 
-size_t sizhead; // length of basic details
 
 typedef struct {
   // the following can be assumed TRUE, they are checked on startup
@@ -342,8 +437,11 @@ typedef struct {
   gboolean has_cdda2wav;
   gboolean has_midistartstop;
   gboolean has_jackd;
+  gboolean has_pulse_audio;
+  gboolean has_xwininfo;
+  gboolean has_gdb;
 
-  // home directory - default location for config file
+  // home directory - default location for config file - locale encoding
   gchar home_dir[256];
 
   // used for returning startup messages from the backend
@@ -352,53 +450,18 @@ typedef struct {
   // plugins
   gboolean has_encoder_plugins;
 
+  gboolean has_python;
+
   gshort cpu_bits;
 
   gchar *myname_full;
   gchar *myname;
 
+  gboolean has_stderr;
+
 } capability;
 
 
-typedef struct {
-  gchar name[51];
-  guint32 audio_codec;
-  // must match with the "acodec" GList in interface.c
-#define AUDIO_CODEC_MP3 0
-#define AUDIO_CODEC_PCM 1
-#define AUDIO_CODEC_MP2 2
-#define AUDIO_CODEC_VORBIS 3
-#define AUDIO_CODEC_AC3 4
-#define AUDIO_CODEC_AAC 5
-#define AUDIO_CODEC_AMR_NB 6
-
-#define AUDIO_CODEC_MAX 31
-  //
-#define AUDIO_CODEC_NONE 32
-#define AUDIO_CODEC_UNKNOWN 33
-
-  guint capabilities;
-
-  /* next two are no longer useful */
-#define CAN_ENCODE_WITH_FRAME_CHANGES 1<<0
-#define CAN_ENCODE_WITH_FPS_CHANGES 1<<1
-
-
-#define CAN_ENCODE_PNG 1<<2
-#define ENCODER_NON_NATIVE 1<<3
-#define HAS_RFX 1<<4
-
-  // current output format
-  gchar of_name[51];
-  gchar of_desc[128];
-  gint of_allowed_acodecs;
-  gchar of_restrict[1024];
-  gchar of_def_ext[16];
-}
-_encoder;
-
-
-
 // some shared structures
 capability *capable;
 
@@ -452,12 +515,13 @@ void resize_play_window (void);
 void kill_play_window (void);
 void make_preview_box (void);
 void add_to_playframe (void);
+GtkTextView *create_output_textview(void);
 
 // dialogs.c
 gboolean do_progress_dialog(gboolean visible, gboolean cancellable, const gchar *text);
-gint do_warning_dialog(const gchar *text);
-gint do_warning_dialog_with_check(const gchar *text, gint warn_mask_number);
-gint do_warning_dialog_with_check_transient(const gchar *text, gint warn_mask_number, GtkWindow *transient);
+gboolean do_warning_dialog(const gchar *text);
+gboolean do_warning_dialog_with_check(const gchar *text, gint warn_mask_number);
+gboolean do_warning_dialog_with_check_transient(const gchar *text, gint warn_mask_number, GtkWindow *transient);
 void do_error_dialog(const gchar *text);
 void do_error_dialog_with_check(const gchar *text, gint warn_mask_number);
 void do_blocking_error_dialog(const gchar *text);
@@ -466,15 +530,17 @@ void do_memory_error_dialog (void);
 void too_many_files(void);
 void tempdir_warning (void);
 void do_audio_import_error(void);
+void do_mt_backup_space_error(lives_mt *, gint memreq_mb);
 
-gint do_clipboard_fps_warning(void);
+gboolean do_clipboard_fps_warning(void);
 void perf_mem_warning(void);
+void do_dvgrab_error(void);
 gboolean do_comments_dialog (void);
 void do_auto_dialog(const gchar *text, gint type);
 void do_encoder_acodec_error (void);
 void do_encoder_sox_error(void);
-gboolean rdet_suggest_values (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gboolean anr, gboolean ignore_fps);
-gboolean do_encoder_restrict_dialog (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gboolean anr);
+gboolean rdet_suggest_values (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gint asigned, gboolean anr, gboolean ignore_fps);
+gboolean do_encoder_restrict_dialog (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gint asigned, gboolean anr);
 void do_keys_window (void);
 void do_mt_keys_window (void);
 void do_messages_window (void);
@@ -488,6 +554,7 @@ gboolean do_set_duplicate_warning (gchar *new_set);
 gboolean do_set_rename_old_layouts_warning(gchar *new_set);
 gboolean do_layout_alter_frames_warning(void);
 gboolean do_layout_alter_audio_warning(void);
+gboolean do_yuv4m_open_warning(void);
 void do_mt_undo_mem_error(void);
 void do_mt_undo_buf_error(void);
 void do_mt_set_mem_error(gboolean has_mt, gboolean trans);
@@ -497,21 +564,39 @@ void do_mt_no_jack_error(lives_mt *mt, gint warn_mask);
 gboolean do_mt_rect_prompt(lives_mt *mt);
 void do_audrate_error_dialog(void);
 gboolean do_event_list_warning(lives_mt *mt);
+void do_nojack_rec_error(void);
+void do_vpp_palette_error (void);
+void do_vpp_fps_error (void);
+void do_decoder_palette_error (void);
+void do_rmem_max_error (gint size);
+gboolean do_original_lost_warning(gchar *fname);
+void do_no_decoder_error(gchar *fname);
+void do_jack_noopen_warn(void);
+void do_jack_noopen_warn2(void);
+void do_file_perm_error(gchar *file_name);
+void do_encoder_img_ftm_error(render_details *rdet);
 
 gboolean process_one (gboolean visible);
-void do_threaded_dialog(gchar *text);
+void do_threaded_dialog(gchar *text, gboolean has_cancel);
 void end_threaded_dialog(void);
 void response_ok (GtkButton *button, gpointer user_data);
 void response_cancel (GtkButton *button, gpointer user_data);
+void pump_io_chan(GIOChannel *iochan);
+
+void do_splash_progress(void);
+
+
+// d_print shortcuts
 void d_print_cancelled(void);
 void d_print_failed(void);
 void d_print_done(void);
+void d_print_file_error_failed(void);
 
 // general
 void do_text_window (const gchar *title, const gchar *text);
 
 // saveplay.c
-void add_file_info(const gchar *check_handle);
+void add_file_info(const gchar *check_handle, gboolean aud_only);
 void save_file_comments (void);
 void reget_afilesize (int fileno);
 void deduce_file(gchar *filename, gdouble start_time, gint end);
@@ -531,6 +616,7 @@ void add_to_recovery_file (gchar *handle);
 void rewrite_recovery_file(gint closed_file); // closed_file < 0 for no closed file
 gboolean check_for_recovery_files (gboolean auto_recover);
 void recover_layout_map(gint numclips);
+gchar *get_deinterlace_string(void);
 
 // saveplay.c backup
 void backup_file(const gchar *filename);
@@ -553,14 +639,19 @@ void close_scrap_file (void);
 
 // main.c
 void catch_sigint(int signum);
-gboolean startup_message_fatal(gpointer msg);
-gboolean startup_message_nonfatal(gpointer msg);
-gboolean startup_message_nonfatal_dismissable(gpointer msg, gint warning_mask);
+gboolean startup_message_fatal(gchar *msg);
+gboolean startup_message_nonfatal(gchar *msg);
+gboolean startup_message_nonfatal_dismissable(gchar *msg, gint warning_mask);
 capability *get_capabilities(void);
+void get_monitors(void);
 void load_start_image(gint frame);
 void load_end_image(gint frame);
 void load_preview_image(gboolean update_always);
-gboolean pull_frame(weed_plant_t *layer, gchar *image_ext, weed_timecode_t tc);
+
+gboolean pull_frame(weed_plant_t *layer, const gchar *image_ext, weed_timecode_t tc);
+gboolean pull_frame_at_size (weed_plant_t *layer, const gchar *image_ext, weed_timecode_t tc, int width, int height, int target_palette);
+GdkPixbuf *pull_gdk_pixbuf_at_size(gint clip, gint frame, const gchar *image_ext, weed_timecode_t tc, gint width, gint height, GdkInterpType interp);
+
 void load_frame_image(gint frame, gint last_frame);
 void sensitize(void);
 void desensitize(void);
@@ -570,34 +661,47 @@ void get_next_free_file(void);
 void switch_to_file(gint old_file, gint new_file);
 void do_quick_switch (gint new_file);
 void resize (gdouble scale);
-GdkPixbuf *lives_scale_simple (gint width, gint height);
+GdkPixbuf *lives_scale_simple (GdkPixbuf *, gint width, gint height);
 void read_file_details(const gchar *file_name, gboolean only_check_for_audio);
-gboolean do_start_messages(gpointer);
+void do_start_messages(void);
 void set_palette_colours (void);
 void set_main_title(const gchar *filename, gint or_untitled_number);
 void set_record (void);
 
+// main.c - in progress
+weed_plant_t *weed_layer_new_from_video(gint fnum, gint frame);
+
 //gui.c
 void fade_background(void);
 void unfade_background(void);
 void block_expose (void);
 void unblock_expose (void);
+void frame_size_update(void);
+void splash_init(void);
+void splash_end(void);
+void splash_msg(const gchar *msg, gdouble pct);
 
 // utils.c
 #ifdef IS_IRIX
 void setenv(const char *name, const char *val, int _xx);
 #endif
 
+LIVES_INLINE float LEFloat_to_BEFloat(float f);
 int lives_10pow(int pow);
+int get_approx_ln(guint val);
 void lives_free(gpointer ptr);
 void lives_free_with_check(gpointer ptr);
+LIVES_INLINE gint myround(gdouble n);
 void get_dirname(gchar *filename);
 void get_basename(gchar *filename);
 void get_filename(gchar *filename);
+gchar *get_extension(const gchar *filename);
 void d_print(const gchar *text);
+void init_clipboard(void);
 void cache_file_contents(gchar *filename);
 gchar *get_val_from_cached_list(const gchar *key, size_t maxlen);
 void get_pref(const gchar *key, gchar *val, gint maxlen);
+void get_pref_utf8(const gchar *key, gchar *val, gint maxlen);
 void get_pref_default(const gchar *key, gchar *val, gint maxlen);
 gboolean get_boolean_pref(const gchar *key);
 gdouble get_double_pref(const gchar *key);
@@ -624,19 +728,30 @@ void colour_equal(GdkColor *c1, const GdkColor *c2);
 void switch_to_int_player(void);
 void switch_to_mplayer(void);
 void switch_aud_to_sox(void);
-void switch_aud_to_jack(void);
+gboolean switch_aud_to_jack(void);
+gboolean switch_aud_to_pulse(void);
 void switch_aud_to_mplayer(void);
 void prepare_to_play_foreign(void);
 gboolean after_foreign_play(void);
-gboolean check_file(const gchar *file_name);  // check if file exists
+gboolean check_file(const gchar *file_name, gboolean check_exists);  // check if file exists
 gboolean check_dir_access (gchar *dir);
 glong get_file_size(int fd);
 glong sget_file_size(gchar *name);
+gboolean ensure_isdir(gchar *fname);
 gchar *ensure_extension(gchar *fname, gchar *ext);
-gdouble calc_variable_time (gint filenum, gint frame);
-gint calc_variable_frame (gint filenum, gdouble time);
+void activate_url_inner(const gchar *link);
+void activate_url (GtkAboutDialog *about, const gchar *link, gpointer data);
+void show_manual_section (const gchar *lang, const gchar *section);
+
+LIVES_INLINE gdouble calc_time_from_frame (gint clip, gint frame);
+LIVES_INLINE gint calc_frame_from_time (gint filenum, gdouble time);  // nearest frame start
+LIVES_INLINE gint calc_frame_from_time2 (gint filenum, gdouble time); // nearest frame end
+LIVES_INLINE gint calc_frame_from_time3 (gint filenum, gdouble time); // nearest frame mid
+
 gboolean check_for_ratio_fps (gdouble fps);
 gdouble get_ratio_fps(gchar *string);
+
+gchar *remove_trailing_zeroes(gdouble val);
 void toggle_button_toggle (GtkToggleButton *tbutton);
 void remove_layout_files(GList *lmap);
 gboolean add_lmap_error(int lerror, gchar *name, gpointer user_data, gint clipno, gint frameno, gdouble atime);
@@ -650,9 +765,13 @@ gboolean check_frame_count(gint idx);
 void get_frame_count(gint idx);
 gint count_resampled_frames (gint in_frames, gdouble orig_fps, gdouble resampled_fps);
 gboolean int_array_contains_value(int *array, int num_elems, int value);
+gboolean check_for_lock_file(gchar *set_name, gint type);
+gint real_clips_available(void);
+void g_list_free_strings(GList *list);
+void gtk_tooltips_copy(GtkWidget *dest, GtkWidget *source);
 
 gboolean create_event_space(gint length_in_eventsb);
-void add_to_recent(const gchar *filename, gdouble start, gint frames);
+void add_to_recent(const gchar *filename, gdouble start, gint frames, const gchar *file_open_params);
 gint verhash (gchar *version);
 void set_undoable (const gchar *what, gboolean sensitive);
 void set_redoable (const gchar *what, gboolean sensitive);
@@ -668,6 +787,7 @@ gchar *g_strappend (gchar *string, gint len, const gchar *new);
 void find_when_to_stop (void);
 void minimise_aspect_delta (gdouble allowed_aspect,gint hblock,gint vblock,gint hsize,gint vsize,gint *width,gint *height);
 
+GList *get_set_list(const gchar *dir);
 void combo_set_popdown_strings (GtkCombo *combo, GList *list);
 
 gchar *subst (gchar *string, gchar *from, gchar *to);
@@ -684,16 +804,31 @@ void set_fg_colour(gint red, gint green, gint blue);
 gboolean label_act_toggle (GtkWidget *, GdkEventButton *, GtkToggleButton *);
 gboolean widget_act_toggle (GtkWidget *, GtkToggleButton *);
 
+void lives_set_cursor_style(gint cstyle, GdkWindow *window);
+
+#define LIVES_CURSOR_NORMAL 0  // must be zero
+#define LIVES_CURSOR_BLOCK 1
+#define LIVES_CURSOR_AUDIO_BLOCK 2
+#define LIVES_CURSOR_BUSY 3
+
+
+
+gchar *text_view_get_text(GtkTextView *textview);
+void text_view_set_text(GtkTextView *textview, const gchar *text);
+
+
 
 // effects.c
 void invalidate_pixel_buffers (void); // TODO - exterminate !
 
 // plugins.c
 GList *get_external_window_hints(lives_rfx_t *rfx);
-gboolean check_encoder_restrictions (gboolean get_extension);
+gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio);
 
 //callbacks.c
 void lives_exit (void);
+void on_check_clicked(void);
+void on_fileread_clicked (GtkFileChooser *fch, gpointer widget);
 
 // paramspecial.c
 gboolean mt_framedraw(lives_mt *, GdkPixbuf *);
@@ -704,12 +839,16 @@ void add_fill_to_box (GtkBox *);
 // rte_window.c
 void refresh_rte_window (void);
 
+// effects-weed.c
+void *w_memcpy  (void *dest, const void *src, size_t n);
 
 // osc.c
 #ifdef ENABLE_OSC
 gboolean lives_osc_init(guint osc_udp_port);
 gint lives_osc_poll(gpointer data);
 void lives_osc_end(void);
+gboolean lives_osc_notify(int msgtype, char *msgstring);
+#include "osc_notify.h"
 #endif
 
 // ldvgrab.c
@@ -721,6 +860,9 @@ void on_open_fw_activate (GtkMenuItem *menuitem, gpointer format);
 
 #endif
 
+int dummyvar;
+char *dummychar;
+
 // inlines
 #define cfile mainw->files[mainw->current_file]
 #define clipboard mainw->files[0]
@@ -747,10 +889,17 @@ void on_open_fw_activate (GtkMenuItem *menuitem, gpointer format);
 #define CLIP_DETAILS_AUTHOR 15
 #define CLIP_DETAILS_COMMENT 16
 #define CLIP_DETAILS_PB_FRAMENO 17
-#define CLIP_DETAILS_CLIPNAME 18
-#define CLIP_DETAILS_HEADER_VERSION 19
-#define CLIP_DETAILS_KEYWORDS 20
+#define CLIP_DETAILS_FILENAME 18
+#define CLIP_DETAILS_CLIPNAME 19
+#define CLIP_DETAILS_HEADER_VERSION 20
+#define CLIP_DETAILS_KEYWORDS 21
 
 #define PREFS_TIMEOUT 10000000 // 10 seconds // TODO !
 
+#define LIVES_TOY_NONE 0
+#define LIVES_TOY_MAD_FRAMES 1
+#define LIVES_TOY_TV 2
+#define LIVES_TOY_AUTOLIVES 2
+#define LIVES_TV_CHANNEL1 "http://www.serverwillprovide.com/sorteal/livestvclips/livestv.ogm"
+
 #endif // #ifndef HAS_MAIN_H
