@@ -1,7 +1,7 @@
 // mainwindow.h
 // LiVES (lives-exe)
-// (c) G. Finch <salsaman@xs4all.nl> 2003
-// Released under the GPL 2.0 or higher
+// (c) G. Finch <salsaman@xs4all.nl> 2003 - 2009
+// Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
 
@@ -10,36 +10,54 @@
 
 #include <pthread.h>
 
+#ifdef ALSA_MIDI
+#include <alsa/asoundlib.h>
+#endif
+
+#ifdef HAVE_PULSE_AUDIO
+#include "pulse.h"
+#endif
+
 // hardware related prefs
 
 // fraction of a second quantisation for event timing; these two must match, and must be multiples of 10>=1000000 !
 // >10**8 is not recommended, since we sometimes store seconds in a gdouble
 #define U_SEC 100000000.
 #define U_SECL (gint64)100000000
-
-// key rpt (millisecs) for kb rpt, should be << FPS_MAX
-// TODO - make pref
-#define KEY_RPT_INTERVAL 40
+#define U_SEC_RATIO (U_SECL/1000000) // how many U_SECs (ticks) in a microsecond
 
 // parameters for resizing the image frames, and for capture
 // TODO - make prefs
 #define V_RESIZE_ADJUST 36
 #define H_RESIZE_ADJUST 22
 
+// vert displacement up from center for sepwin (actual value is half this)
+#define SEPWIN_VADJUST 200
+
 // number of function keys
 #define FN_KEYS 12
 
 // FX keys, 1 - 9 normally
 #define FX_KEYS_PHYSICAL 9
 
-// must be > FX_KEYS_PHYSICAL, and <=64 (number of bits in a 64bit int mask)
+// must be >= FX_KEYS_PHYSICAL, and <=64 (number of bits in a 64bit int mask)
 // (max number of keys accesible through rte window or via OSC)
 #define FX_KEYS_MAX_VIRTUAL 64
 
-// the rest of the keys are accessible through the multitrack renderer
+// the rest of the keys are accessible through the multitrack renderer (must, be > FX_KEYS_MAX_VIRTUAL)
 #define FX_KEYS_MAX 65536
 
 
+// maximum number of external control peripherals
+#define MAX_EXT_CNTL 32
+
+// external control types
+#define EXT_CNTL_NONE 0 // not used
+#define EXT_CNTL_JS 1
+#define EXT_CNTL_MIDI 2
+
+
+
 typedef struct {
   // set in set_palette_colours()
   gint style;
@@ -47,6 +65,9 @@ typedef struct {
 #define STYLE_1 1<<0 // turn on theming if set
 #define STYLE_2 1<<1 // colour the spinbuttons on the front page if set
 #define STYLE_3 1<<2 // style is lightish - allow themeing of widgets with dark text, otherwise use menu bg
+#define STYLE_4 1<<3 // coloured bg for poly window in mt
+#define STYLE_5 1<<4 // drop down menu text col. in mt
+#define STYLE_6 1<<4 // separator col. in mt
 
   GdkColor white;
   GdkColor black;
@@ -56,7 +77,8 @@ typedef struct {
   GdkColor light_red;
   GdkColor grey20;
   GdkColor grey25;
-  GdkColor grey50;
+  GdkColor grey45;
+  GdkColor grey60;
   GdkColor dark_orange;
   GdkColor fade_colour;
   GdkColor normal_back;
@@ -73,8 +95,18 @@ typedef struct {
 } _palette;
 
 
+typedef struct {
+  gint x;
+  gint y;
+  gint width;
+  gint height;
+  GdkScreen *screen;
+} lives_mgeometry_t;
 
 
+// where do we add the builtin tools in the tools menu
+#define RFX_TOOL_MENU_POSN 2
+
 // mainw->
 typedef struct {
   gchar msg[512];
@@ -109,24 +141,17 @@ typedef struct {
   gboolean loop_cont;
   gboolean ping_pong;
   gboolean mute;
-  gboolean must_resize;
+  gboolean must_resize; // fixed playback size in gui; playback plugins have their own fwidth and fheight
   gint audio_start;
   gint audio_end;
 
-  gboolean ext_playback;
-  gboolean ext_keyboard;
-  guint ext_palette;
+  gboolean ext_playback; // using external video playback plugin
+  gboolean ext_keyboard; // keyboard codes must be polled from video playback plugin
 
-  // TODO - use Weed palettes ?
-#define EXTP_UNDEFINED 0
-#define EXTP_RGB24 1
-#define EXTP_UYVY 2
-#define EXTP_YUV420 3
-#define EXTP_RGBA32 4
   gint ptr_x;
   gint ptr_y;
 
-  gdouble fps_measure;
+  gdouble fps_measure; // show fps stats after playback
 
 
   // flags
@@ -141,7 +166,7 @@ typedef struct {
   gboolean record;
 
   gboolean in_fs_preview;
-  gint cancelled;
+  volatile gint cancelled;
 
   // no cancel
 #define CANCEL_NONE 0
@@ -182,22 +207,24 @@ typedef struct {
   // cancelled because of error
 #define CANCEL_ERROR 12
 
+  // cancelled and paused
+#define CANCEL_USER_PAUSED 13
+
   // special cancel for TV toy
 #define CANCEL_KEEP_LOOPING 100
 
   gboolean error;
 
   guint cancel_type;
-#define CANCEL_KILL 0
-#define CANCEL_HUP 1
-#define CANCEL_SOFT 2
+#define CANCEL_KILL 0  // normal - kill background processes working on current clip
+#define CANCEL_SOFT 1 // just cancel in GUI (for keep, etc)
 
   weed_plant_t *event_list;
 
   gshort endian;
 
-  gint pwidth;
-  gint pheight;
+  gint pwidth; // playback width in RGB pixels
+  gint pheight; // playback height
 
   gshort whentostop;
   // which stream end should cause playback to finish ?
@@ -232,7 +259,7 @@ typedef struct {
   // some VJ effects
   gboolean nervous;
 
-  lives_rfx_t *rendered_fx; // external rendered effects plugins
+  lives_rfx_t *rendered_fx;
   gint num_rendered_effects_builtin;
   gint num_rendered_effects_custom;
   gint num_rendered_effects_test;
@@ -255,16 +282,11 @@ typedef struct {
   gdouble blend_factor; // keyboard control parameter
   
   gint blend_file;
-  gint blend_file_step;
+  gint last_blend_file;
 
-  uyvy_frame *uyvy_frame;
-  guchar *yuv_frame;
-  int uyvy_width;
-  int uyvy_height;
+  gint scrap_file; // we throw odd sized frames here when recording in real time; used if a source is a generator or stream
 
-  gint scrap_file;
-
-  // which number file we are playing
+  // which number file we are playing (or -1)
   gint playing_file;
 
   gint pre_src_file; // file we were editing before any ext input started
@@ -315,8 +337,10 @@ typedef struct {
 
   // extra parameters for opening special files
   gchar *file_open_params;
+  gboolean open_deint;
 
   gint last_dprint_file;
+  gboolean no_switch_dprint;
 
   // actual frame being displayed
   gint actual_frame;
@@ -324,21 +348,43 @@ typedef struct {
   // and the audio 'frame' for when we are looping
   gdouble aframeno;
 
+  // ticks are measured in 1/U_SEC of a second (by defalt a tick is 10 nano seconds)
+
   // for the internal player
-  gdouble period;
-  gint64 startticks;
-  gint64 timeout_ticks;
-  gint64 startsecs;
-  gint64 currticks;
-  gint64 deltaticks;
-  gint64 firstticks;
-  gint64 origticks;
-  gboolean size_warn;
-  gboolean noswitch;
+  gdouble period; // == 1./cfile->pb_fps (unless cfile->pb_fps is 0.)
+  gint64 startticks; // effective ticks when last frame was (should have been) displayed
+  gint64 timeout_ticks; // incremented if effect/rendering is paused/previewed
+  gint64 startsecs; // playback start seconds - subtracted from all other ticks to keep numbers smaller
+  gint64 currticks; // current playback ticks : goes from origticks upwards
+  gint64 deltaticks; // deltaticks for scratching
+  gint64 firstticks; // ticks when audio started playing
+  gint64 origticks; // ticks at start of playback
+  gint64 stream_ticks;  // ticks since first frame sent to playback plugin
+
+  gboolean size_warn; // warn the user that incorrectly sized frames were found
+  gboolean noswitch; // set to TRUE during frame load/display operation. If TRUE we should not switch clips, 
+                     // close the current clip, or call load_frame_image()
   gint new_clip;
-  gboolean reinit_effects; // TODO - remove
+
+  gboolean reverse_pb; 
+
+  // TODO - make this a mutex and more finely grained : things we need to block are (clip switches, clip closure, effects on/off, etc)
   gboolean osc_block;
-  gdouble fixed_fps;
+
+  gboolean osc_auto; // bypass user choices automatically
+
+  // encode width and height set externally
+  gint osc_enc_width;
+  gint osc_enc_height;
+
+
+  // fixed fps playback; usually fixed_fpsd==0.
+  gint fixed_fps_numer;
+  gint fixed_fps_denom;
+  gdouble fixed_fpsd; // <=0. means free playback
+
+  // video playback plugin was updated; write settings to a file
+  gboolean write_vpp_file;
 
   gshort scratch;
 #define SCRATCH_NONE 0
@@ -349,7 +395,7 @@ typedef struct {
   gboolean internal_messaging;
   gint (*progress_fn) (gboolean reset);
 
-  gboolean threaded_dialog;
+  volatile gboolean threaded_dialog;
 
   // fx controls
   gdouble fx1_val;
@@ -397,7 +443,10 @@ typedef struct {
   gulong hrule_func;
   gulong toy_func_none;
   gulong toy_func_random_frames;
+  gulong toy_func_lives_tv;
   gulong hnd_id;
+  gulong loop_cont_func;
+  gulong mute_audio_func;
 
   // for jack transport
   gboolean jack_can_stop;
@@ -441,6 +490,8 @@ typedef struct {
   GtkWidget *open_dvd;
   GtkWidget *open_loc;
   GtkWidget *open_yuv4m;
+  GtkWidget *open_lives2lives;
+  GtkWidget *send_lives2lives;
   GtkWidget *open_device_menu;
   GtkWidget *open_device_submenu;
   GtkWidget *open_firewire;
@@ -500,7 +551,6 @@ typedef struct {
 
   GtkWidget *change_speed;
   GtkWidget *capture;
-  GtkWidget *reorder;
   GtkWidget *load_audio;
   GtkWidget *load_cdtrack;
   GtkWidget *eject_cd;
@@ -581,6 +631,8 @@ typedef struct {
   GtkWidget *arrow1;
   GtkWidget *arrow2;
 
+  GdkCursor *cursor;
+
   weed_plant_t *filter_map;
   void ***pchains;
 
@@ -588,10 +640,10 @@ typedef struct {
   gint fixed_height;
   GtkWidget *image274;
   GtkWidget *play_window;
-  GdkPixbuf *frame_image;
-  GdkPixbuf *dbl_size;
+  weed_plant_t *frame_layer;
   GtkWidget *plug1;
   gulong pw_exp_func;
+  gboolean pw_exp_is_blocked;
 
   // frame preview in the separate window
   GtkWidget *preview_box;
@@ -691,6 +743,8 @@ typedef struct {
   GtkWidget *tools_menu;
   GtkWidget *utilities_menu;
   GtkWidget *utilities_submenu;
+  GtkWidget *gens_menu;
+  GtkWidget *gens_submenu;
   GtkWidget *run_test_rfx_submenu;
   GtkWidget *run_test_rfx_menu;
   GtkWidget *custom_effects_menu;
@@ -699,6 +753,8 @@ typedef struct {
   GtkWidget *custom_tools_menu;
   GtkWidget *custom_tools_submenu;
   GtkWidget *custom_tools_separator;
+  GtkWidget *custom_gens_menu;
+  GtkWidget *custom_gens_submenu;
   GtkWidget *custom_utilities_menu;
   GtkWidget *custom_utilities_submenu;
   GtkWidget *custom_utilities_separator;
@@ -709,43 +765,58 @@ typedef struct {
   gint *clip_index;
   gint *frame_index;
 
+  GtkWidget *resize_menuitem;
 
-  gboolean only_close;
+  gboolean only_close; // only close clips - do not exit
 
 #ifdef ENABLE_JACK
   jack_driver_t *jackd; // jack audio playback device
   jack_driver_t *jackd_read; // jack audio recorder device
+#define RT_AUDIO
+#else
+  void *jackd;  // dummy
+  void *jackd_read; // dummy
 #endif
 
+#ifdef HAVE_PULSE_AUDIO
+  pulse_driver_t *pulsed; // pulse audio playback device
+  pulse_driver_t *pulsed_read; // pulse audio recorder device
+#define RT_AUDIO
+#else
+  void *pulsed;
+  void *pulsed_read;
+#endif
 
   // layouts
   GtkTextBuffer *layout_textbuffer; // stores layout errors
   GList *affected_layouts_map; // map of layouts with errors
   GList *current_layouts_map; // map of all layouts for set
 
-  gchar *recovery_file;
+  gchar *recovery_file;  // the filename of our recover file
   gboolean leave_recovery;
 
-  gboolean unordered_blocks;
+  gboolean unordered_blocks; // are we recording unordered blocks ?
 
   gboolean no_exit; // if TRUE, do not exit after saving set
 
-  mt_opts multi_opts;
+  mt_opts multi_opts; // some multitrack options that survive between mt calls
 
   gint rec_aclip;
   gdouble rec_avel;
   gdouble rec_aseek;
 
-  gpointer do_not_free;
+  gpointer do_not_free; // mess with memory so that g_object_unref can be forced not to free() the pixel_data
   GMemVTable alt_vtable;
   void (*free_fn)(gpointer);
 
-  pthread_mutex_t gtk_mutex;
-  pthread_mutex_t interp_mutex;
+  pthread_mutex_t gtk_mutex;  // gtk drawing mutex - used by the threaded dialog
+  pthread_mutex_t interp_mutex;  // interpolation mutex - parameter interpolation must be single threaded
 
-  lives_fx_candidate_t fx_candidates[MAX_FX_CANDIDATE_TYPES];
+  pthread_mutex_t abuf_mutex;  // used to synch audio buffer request count - shared between audio and video threads
 
-  GList *cached_list;
+  lives_fx_candidate_t fx_candidates[MAX_FX_CANDIDATE_TYPES]; // effects which can have candidates from which a delegate is selected (current examples are: audio_volume, resize)
+
+  GList *cached_list;  // cache of preferences or file header file (or NULL)
   FILE *clip_header;
 
   gfloat volume; // audio volume level (for jack)
@@ -760,29 +831,77 @@ typedef struct {
   gint rec_asamps;
   gint rec_signed_endian;
 
-  gboolean no_switchclose_dprint;
-  gchar *any_string;  // text saying "Any", for encoder and output format
+  gboolean suppress_dprint; // tidy up, e.g. by blocking "switched to file..." and "closed file..." messages
+  gchar *any_string;  // localised text saying "Any", for encoder and output format
+  gchar *none_string;  // localised text saying "None", for playback plugin name, etc.
+  gchar *recommended_string;  // localised text saying "recommended", for encoder and output format
+  gchar *disabled_string;  // localised text saying "disabled !", for playback plugin name, etc.
 
   gint opening_frames; // count of frames so far opened, updated after preview (currently)
 
-  gboolean show_procd;
+  gboolean show_procd; // override showing of "processing..." dialog
 
   gboolean block_param_updates; // block visual param changes from updating real values
   gboolean no_interp; // block interpolation (for single frame previews)
 
-  gdouble fd_scale;
+  gdouble fd_scale; // framedraw scale (image scaling downsize factor)
 
-  weed_timecode_t cevent_tc;
+  weed_timecode_t cevent_tc; // timecode of currently processing event
 
-  GdkGC *general_gc;
+  gboolean opening_multi; // flag to indicate multiple file selection
+
+  gboolean record_paused; // pause during recording
+  gboolean record_starting; // start recording at next frame
+
+  gint img_concat_clip;  // when opening multiple, image files can get concatenated here (prefs->concat_images)
+
+  GdkGC *general_gc;  // used for colour drawing
+
+  // rendered generators
+  gboolean gen_to_clipboard;
+  gboolean is_generating;
+
+  gboolean keep_pre;
+
+  GtkWidget *textwidget_focus;
+
+  _vid_playback_plugin *vpp;
+
+  // multi-head support
+  gint nmonitors;
+  lives_mgeometry_t *mgeom;
+
+
+  // external control inputs
+  gboolean ext_cntl[MAX_EXT_CNTL];
+
+  #ifdef ALSA_MIDI
+  snd_seq_t *seq_handle;
+  int alsa_midi_port;
+  #endif
+
+  weed_plant_t *rte_textparm; // send keyboard input to this paramter (usually NULL)
+
+  gint write_abuf; // audio buffer number to write to (for multitrack)
+  volatile gint abufs_to_fill;
+
+  GtkWidget *splash_window;
+  GtkWidget *splash_label;
+  GtkWidget *splash_progress;
+
+  gboolean recoverable_layout;
+
+  gboolean soft_debug; // for testing
+
+  GIOChannel *iochan;
+  GtkTextView *optextview;
 
 } mainwindow;
 
-
+GdkCursor *hidden_cursor;
 
 _palette *palette;
 
-
 typedef struct {
   gulong ins_frame_function;
 
@@ -801,15 +920,9 @@ typedef struct {
 
 } _merge_opts;
 
-
-
-
-
-
 _merge_opts* merge_opts;
 
 // note, we can only have two of these currently, one for rendered effects, one for real time effects
 GtkWidget *fx_dialog[2];
 
-
 #endif // HAS_MAINWINDOW_H
