@@ -1,7 +1,7 @@
 // multitrack.h
 // LiVES
-// (c) G. Finch 2005 <salsaman@xs4all.nl>
-// released under the GNU GPL 2.0 or higher
+// (c) G. Finch 2005 - 2009 <salsaman@xs4all.nl>
+// released under the GNU GPL 3 or later
 // see file ../COPYING for licensing details
 
 // multitrack window
@@ -37,6 +37,8 @@
 
 #define MT_BORDER_WIDTH 20 // width border for window
 
+#define IN_OUT_SEP 10 // min pixel separation for in/out images in poly_box (per image) 
+
 typedef struct _mt lives_mt;
 
 typedef struct _track_rect track_rect;
@@ -52,8 +54,13 @@ struct _mt_opts {
   gshort mouse_mode;
   gboolean show_audio;
   gboolean show_ctx;
-  gboolean ign_aud_sel;
+  gboolean ign_ins_sel;
   gboolean follow_playback;
+  gboolean insert_audio;  // do we insert audio with video ?
+  gboolean pertrack_audio; // do we want pertrack audio ?
+  gboolean audio_bleedthru; // should we allow all audio to bleed thru ?
+  gboolean gang_audio; // gang layer audio volume levels
+  gint back_audio_tracks; // number of backing audio tracks (currently 0 or 1)
 };
 
 #define MOUSE_MODE_MOVE 1
@@ -94,26 +101,36 @@ struct _mt {
   GtkWidget *delblock;
   GtkWidget *clipedit;
   GtkWidget *vpaned;
+  GtkWidget *hpaned;
   GtkWidget *scrollbar;
   GtkWidget *playall;
   GtkWidget *jumpnext;
   GtkWidget *jumpback;
   GtkWidget *render;
+  GtkWidget *prerender_aud;
   GtkWidget *fx_block;
-  GtkWidget *fx_list;
   GtkWidget *fx_delete;
+  GtkWidget *fx_edit;
   GtkWidget *fx_region;
   GtkWidget *fx_region_1;
   GtkWidget *fx_region_2;
+  GtkWidget *fx_region_2av;
+  GtkWidget *fx_region_2v;
+  GtkWidget *fx_region_2a;
   GtkWidget *fx_region_3;
   GtkWidget *move_fx;
   GtkWidget *fx_auto_prev;
+  GtkWidget *fx_auto_prev_box;
   GtkWidget *mm_menuitem;
   GtkWidget *mm_move;
   GtkWidget *mm_select;
   GtkWidget *snapt_menuitem;
   GtkWidget *snapt_on;
   GtkWidget *snapt_off;
+  GtkWidget *insa_menuitem;
+  GtkWidget *insa_menuitemsep;
+  GtkWidget *insa_on;
+  GtkWidget *insa_off;
   GtkWidget *select_track;
   GtkWidget *view_events;
   GtkWidget *view_clips;
@@ -154,6 +171,9 @@ struct _mt {
   GtkWidget *redo;
   GtkWidget *remove_gaps;
   GtkWidget *remove_first_gaps;
+  GtkWidget *split_sel;
+  GtkWidget *ins_gap_sel;
+  GtkWidget *ins_gap_cur;
   GtkWidget *last_filter_map;
   GtkWidget *next_filter_map;
   GtkWidget *fx_list_box;
@@ -181,11 +201,28 @@ struct _mt {
   GtkWidget *eview_button;
   GtkWidget *follow_play;
   GtkWidget *change_max_disp;
+  GtkWidget *add_vid_behind;
+  GtkWidget *add_vid_front;
+  GtkWidget *quit;
+  GtkWidget *fx_params_label;
+  GtkWidget *amixer_button;
+  GtkWidget *view_sel_events;
+  GtkWidget *adjust_start_end;
+  GtkWidget *in_eventbox;
+  GtkWidget *out_eventbox;
+  GtkWidget *start_in_label;
+  GtkWidget *end_out_label;
+  GtkObject *spinbutton_in_adj;
+  GtkObject *spinbutton_out_adj;
+
+  GdkCursor *cursor;
 
   GtkObject *hadjustment;
   GtkWidget *time_scrollbar;
 
-  GtkWidget *audio_draw; // floating audio
+  GList *audio_draws; // list of audio boxes, 0 == backing audio, 1 == track 0 audio, etc.
+
+  GList *audio_vols; // layer volume levels (coarse control) - set in mixer
 
   GtkAccelGroup *accel_group;
   GList *video_draws; // list of video timeline eventboxes, in layer order
@@ -210,6 +247,9 @@ struct _mt {
   gulong snapt_on_func;
   gulong snapt_off_func;
 
+  gulong insa_on_func;
+  gulong insa_off_func;
+
   gulong sepwin_func;
   gulong mute_audio_func;
   gulong loop_cont_func;
@@ -268,7 +308,8 @@ struct _mt {
   gdouble region_init; // point where user pressed the mouse
   gboolean region_updating;
 
-  gboolean is_rendering; // TRUE if we are is the process of rendering to a clip, cf. mainw->is_rendering
+  gboolean is_rendering; // TRUE if we are in the process of rendering/pre-rendering to a clip, cf. mainw->is_rendering
+  gboolean pr_audio; // TRUE if we are in the process of prerendering audio to a clip
 
 #define MAX_DISP_VTRACKS 4
   gint max_disp_vtracks;
@@ -315,7 +356,10 @@ struct _mt {
   gboolean undoable;
   gboolean redoable;
 
-  gboolean changed;
+  gboolean changed; // changed since last saved
+  gboolean auto_changed; // changed since last auto-saved
+
+  int64_t auto_back_time;
 
   // stuff to do with framedraw "special" widgets
   gint inwidth;
@@ -355,8 +399,6 @@ struct _mt {
   gboolean save_all_vals;
   gboolean ignore_load_vals;
 
-#define MT_PRESERVE_VALS 2
-
   gdouble user_fps;
   gint user_width;
   gint user_height;
@@ -374,6 +416,7 @@ struct _mt {
   GList *tl_marks;
 
   weed_plant_t *pb_start_event; // FRAME event from which we start playback
+  weed_plant_t *pb_loop_event; // FRAME event to loop back to (can be different from pb_start_event if we are paused)
 
   weed_plant_t *specific_event; // a pointer to some generally interesting event
 
@@ -381,6 +424,8 @@ struct _mt {
 
   gint cursor_style;
 
+  gboolean is_paused;
+
   /* current size of frame inside playback/preview area */
   gint play_width;
   gint play_height;
@@ -397,6 +442,14 @@ struct _mt {
 
   gboolean is_ready;
 
+  gboolean aud_track_selected;
+
+  gboolean has_audio_file;
+
+  guint idlefunc;
+
+  GList *clip_labels;
+
 };  // lives_mt
 
 
@@ -419,6 +472,7 @@ struct _mt {
 #define MT_UNDO_REMOVE_GAPS 1026
 #define MT_UNDO_DELETE_AUDIO_BLOCK 1027
 #define MT_UNDO_MOVE_AUDIO_BLOCK 1028
+#define MT_UNDO_INSERT_GAP 1029
 
 
 typedef struct {
@@ -428,7 +482,14 @@ typedef struct {
 } mt_undo;
 
 
-
+typedef struct {
+  GtkWidget **ch_sliders;
+  GtkWidget *gang_checkbutton;
+  GtkWidget *inv_checkbutton;
+  gulong *ch_slider_fns;
+  gint nchans;
+  lives_mt *mt;
+} lives_amixer_t;
 
 
 // reasons for track invisibility (bitmap)
@@ -505,15 +566,19 @@ void insert_here_cb (GtkMenuItem *, gpointer mt);
 void insert_audio_here_cb (GtkMenuItem *, gpointer mt);
 void multitrack_end_cb (GtkMenuItem *, gpointer mt);
 void delete_block_cb (GtkMenuItem *, gpointer mt);
+void selblock_cb (GtkMenuItem *, gpointer mt);
 void list_fx_here_cb (GtkMenuItem *, gpointer mt);
+void edit_start_end_cb (GtkMenuItem *menuitem, gpointer user_data);
 
 //// menuitem callbacks
 void on_add_video_track_activate (GtkMenuItem *, gpointer mt);
-void multitrack_playall (GtkWidget *, gpointer mt); // menuitem playall
-void multitrack_insert (GtkMenuItem *, gpointer mt); // menuitem insert
-void multitrack_audio_insert (GtkMenuItem *, gpointer mt); // menuitem insert
-void multitrack_view_events (GtkMenuItem *, gpointer mt); // menuitem insert
+void multitrack_insert (GtkMenuItem *, gpointer mt);
+void multitrack_adj_start_end (GtkMenuItem *, gpointer mt);
+void multitrack_audio_insert (GtkMenuItem *, gpointer mt);
+void multitrack_view_events (GtkMenuItem *, gpointer mt);
+void multitrack_view_sel_events (GtkMenuItem *, gpointer mt);
 void on_render_activate (GtkMenuItem *, gpointer mt);
+void on_prerender_aud_activate (GtkMenuItem *, gpointer mt);
 void on_jumpnext_activate (GtkMenuItem *, gpointer mt);
 void on_jumpback_activate (GtkMenuItem *, gpointer mt);
 void on_delblock_activate (GtkMenuItem *, gpointer mt);
@@ -542,14 +607,16 @@ void select_to_end_time (GtkMenuItem *, gpointer mt);
 void select_all_vid (GtkMenuItem *, gpointer mt);
 void select_no_vid (GtkMenuItem *, gpointer mt);
 void on_split_sel_activate (GtkMenuItem *, gpointer mt);
+void on_split_curr_activate (GtkMenuItem *, gpointer mt);
 void multitrack_undo (GtkMenuItem *, gpointer mt);
 void multitrack_redo (GtkMenuItem *, gpointer mt);
 void on_mt_showkeys_activate (GtkMenuItem *, gpointer);
 void on_mt_list_fx_activate (GtkMenuItem *, gpointer mt);
 void on_mt_delfx_activate (GtkMenuItem *, gpointer mt);
+void on_mt_fx_edit_activate (GtkMenuItem *, gpointer mt);
 void mt_view_audio_toggled (GtkMenuItem *, gpointer mt);
 void mt_view_ctx_toggled (GtkMenuItem *, gpointer mt);
-void mt_ign_aud_sel_toggled (GtkMenuItem *, gpointer mt);
+void mt_ign_ins_sel_toggled (GtkMenuItem *, gpointer mt);
 void mt_change_max_disp_tracks (GtkMenuItem *, gpointer mt);
 
 // event_list functions
@@ -557,16 +624,19 @@ weed_plant_t *add_blank_frames_up_to (weed_plant_t *event_list, weed_plant_t *st
 
 /// track functions
 void on_cback_audio_activate (GtkMenuItem *, gpointer mt);
-void add_audio_track (lives_mt *);
+GtkWidget *add_audio_track (lives_mt *, gint trackno, gboolean behind);
 void add_video_track_behind (GtkMenuItem *, gpointer mt);
 void add_video_track_front (GtkMenuItem *, gpointer mt);
 void delete_video_track(lives_mt *, gint layer, gboolean full);
 void delete_audio_track(lives_mt *mt, GtkWidget *eventbox, gboolean full);
+void delete_audio_tracks(lives_mt *mt, GList *list, gboolean full);
 void remove_gaps (GtkMenuItem *, gpointer mt);
 void remove_first_gaps (GtkMenuItem *, gpointer mt);
+void on_insgap_sel_activate (GtkMenuItem *, gpointer mt);
+void on_insgap_cur_activate (GtkMenuItem *, gpointer mt);
 void on_split_activate (GtkMenuItem *, gpointer mt);
 void scroll_tracks (lives_mt *mt);
-gboolean atrack_arrow_pressed (GtkWidget *ahbox, GdkEventButton *, gpointer mt);
+gboolean track_arrow_pressed (GtkWidget *ahbox, GdkEventButton *, gpointer mt);
 
 // track mouse movement
 gboolean on_track_click (GtkWidget *eventbox, GdkEventButton *, gpointer mt);
@@ -619,7 +689,8 @@ void mouse_mode_context(lives_mt *);
 void do_sel_context (lives_mt *);
 void do_fx_list_context (lives_mt *mt, gint fxcount);
 
-// playback animation
+// playback / animation
+void multitrack_playall (lives_mt *mt);
 void animate_multitrack (lives_mt *);
 void unpaint_line(lives_mt *, GtkWidget *eventbox);
 void unpaint_lines(lives_mt *);
@@ -669,13 +740,19 @@ void add_markers(lives_mt *, weed_plant_t *event_list);
 void remove_markers(weed_plant_t *event_list);
 
 
+// auto backup
+void recover_layout(GtkButton *, gpointer);
+void recover_layout_cancelled(GtkButton *, gpointer user_data);
+
+
 // internal functions
 void mouse_select_end(GtkWidget *widget, lives_mt *mt);
 
 
 // misc
 void mt_change_disp_tracks_ok (GtkButton *, gpointer user_data);
-
+void mt_swap_play_pause (lives_mt *mt, gboolean put_pause);
+void amixer_show (GtkButton *button, gpointer user_data);
 
 
 /* default to warn about */
@@ -694,10 +771,4 @@ void mt_change_disp_tracks_ok (GtkButton *, gpointer user_data);
 #define LMAP_INFO_SETNAME_CHANGED 16
 
 
-#define LIVES_CURSOR_NORMAL 0  // must be zero
-#define LIVES_CURSOR_BLOCK 1
-#define LIVES_CURSOR_AUDIO_BLOCK 2
-#define LIVES_CURSOR_BUSY 3
-
-
 #endif
