@@ -1,14 +1,16 @@
-
 // rte_window.c
 // LiVES (lives-exe)
 // (c) G. Finch 2005
-// released under the GNU GPL 2.0 or higher
+// released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
 
 #include <errno.h>
 #include <sys/stat.h>
 
+#include "../libweed/weed.h"
+#include "../libweed/weed-host.h"
+
 #include "main.h"
 
 #include "support.h"
@@ -90,7 +92,7 @@ gboolean on_clear_all_clicked (GtkButton *button, gpointer user_data) {
     for (j=modes-1;j>=0;j--) {
       weed_delete_effectkey (i+1,j);
       if (rte_window!=NULL) {
-	gtk_entry_set_text (GTK_ENTRY(combo_entries[i*modes+j]),g_strdup(""));
+	gtk_entry_set_text (GTK_ENTRY(combo_entries[i*modes+j]),"");
 	type_label_set_text(i,j);
       }
     }
@@ -108,22 +110,24 @@ on_save_keymap_clicked (GtkButton *button, gpointer user_data) {
   int modes=rte_getmodespk();
   int i,j;
   FILE *kfile;
-  gchar *msg;
+  gchar *msg,*tmp;
   GList *list=NULL;
   gboolean update=FALSE;
 
-  const gchar *keymap_file=g_strdup_printf("%s/%s/default.keymap",capable->home_dir,LIVES_CONFIG_DIR);
+  const gchar *keymap_file=g_strdup_printf("%s/%sdefault.keymap",capable->home_dir,LIVES_CONFIG_DIR);
 
   if (button!=NULL) {
     if (!do_warning_dialog_with_check_transient((_("\n\nClick 'OK' to save this keymap as your default\n\n")),0,GTK_WINDOW(rte_window))) return FALSE;
 
-    d_print (g_strdup_printf(_("Saving keymap to %s\n"),keymap_file));
+    d_print ((tmp=g_strdup_printf(_("Saving keymap to %s\n"),keymap_file)));
+    g_free(tmp);
   }
   else {
     update=TRUE;
     list=(GList *)user_data;
     if (list==NULL) return FALSE;
-    d_print (g_strdup_printf(_("\nUpdating keymap file %s..."),keymap_file));
+    d_print ((tmp=g_strdup_printf(_("\nUpdating keymap file %s..."),keymap_file)));
+    g_free(tmp);
   }
 
   if (!(kfile=fopen(keymap_file,"w"))) {
@@ -141,7 +145,7 @@ on_save_keymap_clicked (GtkButton *button, gpointer user_data) {
   if (!update) {
     for (i=1;i<=prefs->rte_keys_virtual;i++) {
       for (j=0;j<modes;j++) {
-	if (rte_keymode_valid(i,j)) {
+	if (rte_keymode_valid(i,j,TRUE)) {
 	  fputs(g_strdup_printf("%d|Weed%s\n",i,make_weed_hashname(rte_keymode_get_filter_idx(i,j))),kfile);
 	}
       }
@@ -167,11 +171,11 @@ on_save_rte_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
   gchar *msg;
 
   if (prefs->fxdefsfile==NULL) {
-    prefs->fxdefsfile=g_strdup_printf("%s/%s/fxdefs",capable->home_dir,LIVES_CONFIG_DIR);
+    prefs->fxdefsfile=g_strdup_printf("%s/%sfxdefs",capable->home_dir,LIVES_CONFIG_DIR);
   }
 
   if (prefs->fxsizesfile==NULL) {
-    prefs->fxsizesfile=g_strdup_printf("%s/%s/fxsizes",capable->home_dir,LIVES_CONFIG_DIR);
+    prefs->fxsizesfile=g_strdup_printf("%s/%sfxsizes",capable->home_dir,LIVES_CONFIG_DIR);
   }
 
   msg=g_strdup_printf(_("Saving real time effect defaults to %s..."),prefs->fxdefsfile);
@@ -186,9 +190,9 @@ on_save_rte_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
     return;
   }
   
-  msg=g_strdup("LiVES filter defaults file version 1\n");
+  msg=g_strdup("LiVES filter defaults file version 1.1\n");
   
-  write(fd,msg,strlen(msg));
+  dummyvar=write(fd,msg,strlen(msg));
   g_free(msg);
 
   numfx=rte_get_numfilters();
@@ -207,7 +211,7 @@ on_save_rte_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
 
   msg=g_strdup("LiVES generator default sizes file version 2\n");
   
-  write(fd,msg,strlen(msg));
+  dummyvar=write(fd,msg,strlen(msg));
   g_free(msg);
 
   for (i=0;i<numfx;i++) write_generator_sizes(fd,i);
@@ -228,7 +232,7 @@ void load_rte_defs (void) {
   ssize_t bytes;
 
   if (prefs->fxdefsfile==NULL) {
-    prefs->fxdefsfile=g_strdup_printf("%s/%s/fxdefs",capable->home_dir,LIVES_CONFIG_DIR);
+    prefs->fxdefsfile=g_strdup_printf("%s/%sfxdefs",capable->home_dir,LIVES_CONFIG_DIR);
   }
 
   if (g_file_test(prefs->fxdefsfile,G_FILE_TEST_EXISTS)) {
@@ -242,7 +246,7 @@ void load_rte_defs (void) {
       d_print(msg2);
       g_free(msg2);
       
-      msg=g_strdup("LiVES filter defaults file version 1\n");
+      msg=g_strdup("LiVES filter defaults file version 1.1\n");
       buf=g_malloc(strlen(msg));
       bytes=read(fd,buf,strlen(msg));
       
@@ -250,7 +254,7 @@ void load_rte_defs (void) {
 	read_filter_defaults(fd);
 	d_print_done();
       }
-      else d_print(_("error in file. Failed.\n"));
+      else d_print_file_error_failed();
       
       close(fd);
       
@@ -260,7 +264,7 @@ void load_rte_defs (void) {
   }
 
   if (prefs->fxsizesfile==NULL) {
-    prefs->fxsizesfile=g_strdup_printf("%s/%s/fxsizes",capable->home_dir,LIVES_CONFIG_DIR);
+    prefs->fxsizesfile=g_strdup_printf("%s/%sfxsizes",capable->home_dir,LIVES_CONFIG_DIR);
   }
 
   if (g_file_test(prefs->fxsizesfile,G_FILE_TEST_EXISTS)) {
@@ -284,7 +288,7 @@ void load_rte_defs (void) {
 	d_print_done();
       }
     }
-    else d_print(_("error in file. Failed.\n"));
+    else d_print_file_error_failed();
     
     close(fd);
     
@@ -298,13 +302,12 @@ void load_rte_defs (void) {
 
 
 
-gboolean 
-on_load_keymap_clicked (GtkButton *button, gpointer user_data) {
+gboolean on_load_keymap_clicked (GtkButton *button, gpointer user_data) {
   // quick and dirty implementation
   int modes=rte_getmodespk();
   int i;
   FILE *kfile;
-  gchar *msg;
+  gchar *msg,*tmp;
   gint key,mode;
   gchar buff[65536];
   size_t linelen;
@@ -316,10 +319,12 @@ on_load_keymap_clicked (GtkButton *button, gpointer user_data) {
   gchar *line=NULL;
   gchar *whashname;
 
-  gchar *keymap_file=g_strdup_printf("%s/%s/default.keymap",capable->home_dir,LIVES_CONFIG_DIR);
+  gchar *keymap_file=g_strdup_printf("%s/%sdefault.keymap",capable->home_dir,LIVES_CONFIG_DIR);
   gint count;
 
-  d_print (g_strdup_printf(_("Loading default keymap from %s..."),keymap_file));
+  msg=g_strdup_printf(_("Loading default keymap from %s..."),keymap_file);
+  d_print(msg);
+  g_free(msg);
 
   if (!(kfile=fopen (keymap_file,"r"))) {
     msg=g_strdup_printf (_("\n\nUnable to read from keymap file\n%s\nError code %d\n"),keymap_file,errno);
@@ -356,15 +361,13 @@ on_load_keymap_clicked (GtkButton *button, gpointer user_data) {
   if (!strcmp(g_list_nth_data(list,0),"LiVES keymap file version 2")||!strcmp(g_list_nth_data(list,0),"LiVES keymap file version 1")) update=1;
   if (!strcmp(g_list_nth_data(list,0),"LiVES keymap file version 3")) update=2;
   
-  line=NULL;
-
   for (i=1;i<g_list_length(list);i++) {
     gchar **array;
-    if (line!=NULL) g_free(line);
     line=(gchar *)g_list_nth_data(list,i);
-
+    
     if (get_token_count(line,'|')<2) {
-      d_print(g_strdup_printf(_("Invalid line %d in %s\n"),i,keymap_file));
+      d_print((tmp=g_strdup_printf(_("Invalid line %d in %s\n"),i,keymap_file)));
+      g_free(tmp);
       continue;
     }
 
@@ -402,13 +405,15 @@ on_load_keymap_clicked (GtkButton *button, gpointer user_data) {
     }
 
     if (key<1||key>prefs->rte_keys_virtual) {
-      d_print(g_strdup_printf(_("Invalid key %d in %s\n"),key,keymap_file));
+      d_print((tmp=g_strdup_printf(_("Invalid key %d in %s\n"),key,keymap_file)));
+      g_free(tmp);
       notfound=TRUE;
       continue;
     }
 
     if (strncmp(hashname,"Weed",4)||strlen(hashname)<5) {
-      d_print(g_strdup_printf(_("Invalid effect %s in %s\n"),hashname,keymap_file));
+      d_print((tmp=g_strdup_printf(_("Invalid effect %s in %s\n"),hashname,keymap_file)));
+      g_free(tmp);
       notfound=TRUE;
       continue;
     }
@@ -421,24 +426,29 @@ on_load_keymap_clicked (GtkButton *button, gpointer user_data) {
       continue;
     }
     if (mode==-2){
-      d_print(g_strdup_printf(_("This version of LiVES cannot mix generators/non-generators on the same key (%d) !\n"),key));
+      d_print((tmp=g_strdup_printf(_("This version of LiVES cannot mix generators/non-generators on the same key (%d) !\n"),key)));
+      g_free(tmp);
       continue;
     }
     if (mode==-3){
-      d_print(g_strdup_printf(_("Too many effects bound to key %d.\n"),key));
+      d_print((tmp=g_strdup_printf(_("Too many effects bound to key %d.\n"),key)));
+      g_free(tmp);
       continue;
     }
     else if (rte_window!=NULL) {
-      gtk_entry_set_text (GTK_ENTRY(combo_entries[(key-1)*modes+mode]),rte_keymode_get_filter_name(key,mode));
+      gtk_entry_set_text (GTK_ENTRY(combo_entries[(key-1)*modes+mode]),(tmp=rte_keymode_get_filter_name(key,mode)));
+      g_free(tmp);
       type_label_set_text(key-1,mode);
     }
   }
-  if (line!=NULL) g_free(line);
+
+  g_list_free_strings(list);
   g_list_free(list);
 
   if (update>0) {
     d_print(_("update required.\n"));
     on_save_keymap_clicked(NULL,new_list);
+    g_list_free_strings(new_list);
     g_list_free(new_list);
     on_load_keymap_clicked(NULL,NULL);
   }
@@ -484,7 +494,7 @@ on_rte_info_clicked (GtkButton *button, gpointer user_data) {
 
   ////////////////////////
 
-  if (!rte_keymode_valid(key+1,mode)) return;
+  if (!rte_keymode_valid(key+1,mode,TRUE)) return;
 
   plugin_name=rte_keymode_get_plugin_name(key+1,mode);
   filter=rte_keymode_get_filter(key+1,mode);
@@ -563,6 +573,13 @@ on_rte_info_clicked (GtkButton *button, gpointer user_data) {
 
   textview = gtk_text_view_new ();
   gtk_widget_show (textview);
+
+  if (palette->style&STYLE_1) {
+    gtk_widget_modify_text(textview, GTK_STATE_NORMAL, &palette->normal_fore);
+    gtk_widget_modify_base(textview, GTK_STATE_NORMAL, &palette->normal_back);
+  }
+
+
   textbuffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
   
   gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
@@ -606,7 +623,7 @@ void check_clear_all_button (void) {
 
   for (i=0;i<prefs->rte_keys_virtual;i++) {
     for (j=modes-1;j>=0;j--) {
-      if (rte_keymode_valid(i+1,j)) hasone=TRUE;
+      if (rte_keymode_valid(i+1,j,TRUE)) hasone=TRUE;
     }
   }
 
@@ -640,18 +657,17 @@ on_clear_clicked (GtkButton *button, gpointer user_data) {
     type_label_set_text(key,i);
   }
   idx++;
-  gtk_entry_set_text (GTK_ENTRY(combo_entries[idx]),g_strdup(""));
+  gtk_entry_set_text (GTK_ENTRY(combo_entries[idx]),"");
   type_label_set_text(key,i);
 
-  if (!rte_keymode_valid(key+1,0)) rtew_set_keych(key,FALSE);
+  if (!rte_keymode_valid(key+1,0,TRUE)) rtew_set_keych(key,FALSE);
 
   check_clear_all_button();
 
 }
 
 
-void 
-on_params_clicked (GtkButton *button, gpointer user_data) {
+void on_params_clicked (GtkButton *button, gpointer user_data) {
   gint idx=GPOINTER_TO_INT(user_data);
   int modes=rte_getmodespk();
   gint key=(gint)(idx/modes);
@@ -669,6 +685,7 @@ on_params_clicked (GtkButton *button, gpointer user_data) {
   }
 
   rfx=weed_to_rfx(inst,FALSE);
+  rfx->min_frames=-1;
   keyw=key;
   modew=mode;
   on_render_fx_pre_activate(NULL,rfx);
@@ -680,32 +697,19 @@ on_params_clicked (GtkButton *button, gpointer user_data) {
 
 gboolean
 on_rtew_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data) {
-  GList *list;
 
   if (hash_list!=NULL) {
-    list=hash_list;
-    while (list!=NULL) {
-      g_free(list->data);
-      list=list->next;
-    }
+    g_list_free_strings(hash_list);
     g_list_free(hash_list);
   }
 
   if (name_list!=NULL) {
-    list=name_list;
-    while (list!=NULL) {
-      g_free(list->data);
-      list=list->next;
-    }
+    g_list_free_strings(name_list);
     g_list_free(name_list);
   }
 
   if (name_type_list!=NULL) {
-    list=name_type_list;
-    while (list!=NULL) {
-      g_free(list->data);
-      list=list->next;
-    }
+    g_list_free_strings(name_type_list);
     g_list_free(name_type_list);
   }
 
@@ -753,6 +757,7 @@ fx_changed (GtkItem *item, gpointer user_data) {
   gchar *hashname1=g_strdup(g_object_get_data(G_OBJECT(item),"hashname"));
   gchar *hashname2=g_strdup(g_object_get_data(G_OBJECT(combo_entries[key_mode]),"hashname"));
   gint error;
+  gchar *tmp;
 
   int i;
 
@@ -762,9 +767,9 @@ fx_changed (GtkItem *item, gpointer user_data) {
     return;
   }
 
-  if (!rte_keymode_valid(key+1,mode)) {
+  if (!rte_keymode_valid(key+1,mode,TRUE)) {
     for (i=mode-1;i>=0;i--) {
-      if (rte_keymode_valid(key+1,i)) {
+      if (rte_keymode_valid(key+1,i,TRUE)) {
 	mode=i+1;
 	i=-1;
       }
@@ -775,11 +780,15 @@ fx_changed (GtkItem *item, gpointer user_data) {
   gtk_widget_grab_focus (combo_entries[key_mode]);
 
   if ((error=rte_switch_keymode (key+1, mode, hashname1))<0) {
-    gtk_entry_set_text (GTK_ENTRY (combo_entries[key_mode]),rte_keymode_get_filter_name(key+1,mode));
+    gtk_entry_set_text (GTK_ENTRY (combo_entries[key_mode]),(tmp=rte_keymode_get_filter_name(key+1,mode)));
+    g_free(tmp);
 
     // this gets called twice, unfortunately...may be a bug in gtk+
     if (error==-2) do_mix_error();
-    if (error==-1) d_print(g_strdup_printf(_("LiVES could not locate the effect %s.\n"),rte_keymode_get_filter_name(key+1,mode)));
+    if (error==-1) {
+      d_print((tmp=g_strdup_printf(_("LiVES could not locate the effect %s.\n"),rte_keymode_get_filter_name(key+1,mode))));
+      g_free(tmp);
+    }
     g_free(hashname1);
     g_free(hashname2);
     return;
@@ -799,17 +808,19 @@ fx_changed (GtkItem *item, gpointer user_data) {
 
 
 
-GtkWidget*
-create_rte_window (void)
-{
-  int i,j,idx,fx_idx;
+GtkWidget * create_rte_window (void) {
+  int i,j,idx,fx_idx,error;
 
-  gint winsize_h=mainw->scr_width-100;
-  gint winsize_v=mainw->scr_height-200;
+  gint winsize_h;
+  gint winsize_v;
+
+  gint scr_width,scr_height;
 
   GtkWidget *rte_window;
   GtkWidget *table;
   GtkWidget *hbox;
+  GtkWidget *hbox2;
+  GtkWidget *eventbox;
   GSList *mode_group = NULL;
   GSList *grab_group = NULL;
   GtkWidget *vbox;
@@ -828,8 +839,23 @@ create_rte_window (void)
   GList *list;
   GtkWidget *item;
 
+  gchar *tmp;
+
   ///////////////////////////////////////////////////////////////////////////
 
+
+  if (prefs->gui_monitor==0) {
+    scr_width=mainw->scr_width;
+    scr_height=mainw->scr_height;
+  }
+  else {
+    scr_width=mainw->mgeom[prefs->gui_monitor-1].width;
+    scr_height=mainw->mgeom[prefs->gui_monitor-1].height;
+  }
+
+  winsize_h=scr_width-100;
+  winsize_v=scr_height-200;
+
   key_checks=(GtkWidget **)g_malloc((prefs->rte_keys_virtual)*sizeof(GtkWidget *));
   key_grabs=(GtkWidget **)g_malloc((prefs->rte_keys_virtual)*sizeof(GtkWidget *));
   mode_radios=(GtkWidget **)g_malloc((prefs->rte_keys_virtual)*modes*sizeof(GtkWidget *));
@@ -871,26 +897,70 @@ create_rte_window (void)
     gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
     
     label = gtk_label_new (g_strdup_printf(_("Ctrl-%d"),i+1));
+    if (palette->style&STYLE_1) {
+      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
+    }
     
     gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, FALSE, 0);
 
-    key_checks[i] = gtk_check_button_new_with_label (_("Key active"));
+    key_checks[i] = gtk_check_button_new ();
+    eventbox=gtk_event_box_new();
+    label=gtk_label_new (_("Key active"));
+
+    gtk_container_add(GTK_CONTAINER(eventbox),label);
+    g_signal_connect (GTK_OBJECT (eventbox), "button_press_event",
+		      G_CALLBACK (label_act_toggle),
+		      key_checks[i]);
+
+    if (palette->style&STYLE_1) {
+      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
+      gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
+      gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
+    }
+    
+    hbox2 = gtk_hbox_new (FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), hbox2, FALSE, FALSE, 10);
+    gtk_box_pack_start (GTK_BOX (hbox2), key_checks[i], FALSE, FALSE, 10);
+    gtk_box_pack_start (GTK_BOX (hbox2), eventbox, FALSE, FALSE, 10);
+    GTK_WIDGET_SET_FLAGS (key_checks[i], GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
+
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(key_checks[i]),mainw->rte&(GU641<<i));
 
     ch_fns[i]=g_signal_connect_after (GTK_OBJECT (key_checks[i]), "toggled",
                       G_CALLBACK (rte_on_off_callback_hook),GINT_TO_POINTER (i+1));
 
-    gtk_box_pack_start (GTK_BOX (hbox), key_checks[i], FALSE, FALSE, 0);
-    
-    key_grabs[i] = gtk_radio_button_new_with_label (grab_group, _("Key grab"));
-    grab_group = gtk_radio_button_group (GTK_RADIO_BUTTON (key_grabs[i]));
+
+
+    hbox2 = gtk_hbox_new (FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), hbox2, FALSE, FALSE, 10);
+
+    key_grabs[i]=gtk_radio_button_new(grab_group);
+    grab_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (key_grabs[i]));
+
+    gtk_box_pack_start (GTK_BOX (hbox2), key_grabs[i], FALSE, FALSE, 10);
+
+    label=gtk_label_new_with_mnemonic (_ ("Key grab"));
+    gtk_label_set_mnemonic_widget (GTK_LABEL (label),key_grabs[i]);
+
+    eventbox=gtk_event_box_new();
+    gtk_container_add(GTK_CONTAINER(eventbox),label);
+    gtk_tooltips_set_tip (mainw->tooltips, eventbox, _("Grab keyboard for this effect key"), NULL);
+    gtk_tooltips_copy(key_grabs[i],eventbox);
+    g_signal_connect (GTK_OBJECT (eventbox), "button_press_event",
+		      G_CALLBACK (label_act_toggle),
+		      key_grabs[i]);
+    if (palette->style&STYLE_1) {
+      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
+      gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
+      gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
+    }
+    gtk_box_pack_start (GTK_BOX (hbox2), eventbox, FALSE, FALSE, 10);
+
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(key_grabs[i]),mainw->rte_keys==i);
 
     gr_fns[i]=g_signal_connect_after (GTK_OBJECT (key_grabs[i]), "toggled",
 				      G_CALLBACK (grabkeys_callback_hook),GINT_TO_POINTER (i));
 
-    gtk_box_pack_start (GTK_BOX (hbox), key_grabs[i], FALSE, FALSE, 0);
-
     mode_group=NULL;
 
     clear_all_button = gtk_button_new_with_mnemonic (_("_Clear all effects"));
@@ -903,10 +973,30 @@ create_rte_window (void)
 			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
       gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
 
-      mode_radios[idx] = gtk_radio_button_new_with_label (mode_group, _("Mode active"));
-      mode_group = gtk_radio_button_group (GTK_RADIO_BUTTON (mode_radios[idx]));
 
-      gtk_box_pack_start (GTK_BOX (hbox), mode_radios[idx], FALSE, FALSE, 0);
+      hbox2 = gtk_hbox_new (FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (hbox), hbox2, FALSE, FALSE, 10);
+      
+      mode_radios[idx]=gtk_radio_button_new(mode_group);
+      mode_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (mode_radios[idx]));
+      
+      gtk_box_pack_start (GTK_BOX (hbox2), mode_radios[idx], FALSE, FALSE, 10);
+
+      label=gtk_label_new_with_mnemonic (_ ("Mode active"));
+      gtk_label_set_mnemonic_widget (GTK_LABEL (label),mode_radios[idx]);
+
+      eventbox=gtk_event_box_new();
+      gtk_container_add(GTK_CONTAINER(eventbox),label);
+      g_signal_connect (GTK_OBJECT (eventbox), "button_press_event",
+			G_CALLBACK (label_act_toggle),
+			mode_radios[idx]);
+      if (palette->style&STYLE_1) {
+	gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
+	gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
+	gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
+      }
+      gtk_box_pack_start (GTK_BOX (hbox2), eventbox, FALSE, FALSE, 10);
+
 
       if (rte_key_getmode(i+1)==j) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mode_radios[idx]),TRUE);
 
@@ -914,6 +1004,9 @@ create_rte_window (void)
 					       G_CALLBACK (rtemode_callback_hook),GINT_TO_POINTER (idx));
 
       type_labels[idx] = gtk_label_new ("");
+      if (palette->style&STYLE_1) {
+	gtk_widget_modify_fg(type_labels[idx], GTK_STATE_NORMAL, &palette->normal_fore);
+      }
 
       info_buttons[idx] = gtk_button_new_with_label (_("Info"));
       param_buttons[idx] = gtk_button_new_with_label (_("Set Parameters"));
@@ -927,6 +1020,9 @@ create_rte_window (void)
       gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
       nlabels[idx] = gtk_label_new (_("Effect name:"));
+      if (palette->style&STYLE_1) {
+	gtk_widget_modify_fg(nlabels[idx], GTK_STATE_NORMAL, &palette->normal_fore);
+      }
 
       gtk_box_pack_start (GTK_BOX (hbox), nlabels[idx], FALSE, FALSE, 0);
 
@@ -941,20 +1037,29 @@ create_rte_window (void)
       list=name_type_list;
 
       while (list!=NULL) {
+	weed_plant_t *filter=get_weed_filter(weed_get_idx_for_hashname(g_list_nth_data(hash_list,fx_idx),FALSE));
+	if ((enabled_in_channels(filter,FALSE)>1&&!has_video_chans_in(filter,FALSE))||(weed_plant_has_leaf(filter,"host_menu_hide")&&weed_get_boolean_value(filter,"host_menu_hide",&error)==WEED_TRUE)) {
+	  list = list->next;
+	  fx_idx++;
+	  continue; // skip audio transitions and hidden entries
+	}
 	item = gtk_list_item_new_with_label ((gchar *) list->data);
 	gtk_widget_show(item);
 	gtk_container_add (GTK_CONTAINER (GTK_COMBO(combo)->list), item);
 	g_object_set_data (G_OBJECT(item),"hashname",g_list_nth_data(hash_list,fx_idx));
 	gtk_combo_set_item_string(GTK_COMBO(combo),GTK_ITEM(item),g_list_nth_data(name_list,fx_idx));
+
 	if (fx_idx==rte_keymode_get_filter_idx(i+1,j)) {
 	  g_object_set_data (G_OBJECT(combo_entries[idx]),"hashname",g_list_nth_data(hash_list,fx_idx));
 	}
+
 	g_signal_connect (G_OBJECT(item), "select", G_CALLBACK (fx_changed), GINT_TO_POINTER(idx));
 	list = list->next;
 	fx_idx++;
       }
 
-      gtk_entry_set_text (GTK_ENTRY (combo_entries[idx]),rte_keymode_get_filter_name(i+1,j));
+      gtk_entry_set_text (GTK_ENTRY (combo_entries[idx]),(tmp=rte_keymode_get_filter_name(i+1,j)));
+      g_free(tmp);
       gtk_entry_set_editable (GTK_ENTRY (combo_entries[idx]), FALSE);
 
       hbox = gtk_hbox_new (FALSE, 12);
@@ -968,7 +1073,7 @@ create_rte_window (void)
 
       g_signal_connect (GTK_OBJECT (param_buttons[idx]), "clicked",
 			G_CALLBACK (on_params_clicked),GINT_TO_POINTER (idx));
-
+      
       type_label_set_text(i,j);
 
       gtk_box_pack_start (GTK_BOX (hbox), type_labels[idx], FALSE, FALSE, 0);
@@ -979,6 +1084,7 @@ create_rte_window (void)
       }
   }
 
+
   scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   
@@ -1000,7 +1106,7 @@ create_rte_window (void)
   gtk_container_add (GTK_CONTAINER (rte_window), top_vbox);
 
   hbuttonbox = gtk_hbutton_box_new ();
-  gtk_box_pack_start (GTK_BOX (top_vbox), hbuttonbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (top_vbox), hbuttonbox, FALSE, TRUE, 20);
   gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox), DEF_BUTTON_WIDTH, -1);
   gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_SPREAD);
 
@@ -1049,7 +1155,17 @@ create_rte_window (void)
 
   gtk_widget_show_all(rte_window);
   gtk_widget_hide(dummy_radio);
-  gtk_window_maximize (GTK_WINDOW(rte_window));
+
+  if (prefs->gui_monitor!=0) {
+    gint xcen=mainw->mgeom[prefs->gui_monitor-1].x+(mainw->mgeom[prefs->gui_monitor-1].width-rte_window->allocation.width)/2;
+    gint ycen=mainw->mgeom[prefs->gui_monitor-1].y+(mainw->mgeom[prefs->gui_monitor-1].height-rte_window->allocation.height)/2;
+    gtk_window_set_screen(GTK_WINDOW(rte_window),mainw->mgeom[prefs->gui_monitor-1].screen);
+    gtk_window_move(GTK_WINDOW(rte_window),xcen,ycen);
+  }
+
+  if (prefs->open_maximised) {
+    gtk_window_maximize (GTK_WINDOW(rte_window));
+  }
   return rte_window;
 }
 
@@ -1164,13 +1280,12 @@ void update_pwindow (gint key, gint i, GList *list) {
     if (key==keyw) {
       if ((inst=rte_keymode_get_instance(key+1,modew))==NULL) return;
       rfx=g_object_get_data(G_OBJECT(fx_dialog[1]),"rfx");
-      set_param_from_list(list,&rfx->params[i],0,TRUE);
+      set_param_from_list(list,&rfx->params[i],0,TRUE,TRUE);
     }
   }
 }
 
-void 
-rte_set_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
+void rte_set_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
   gint idx=GPOINTER_TO_INT(user_data);
   weed_plant_t *filter=get_weed_filter(idx);
   lives_rfx_t *rfx;
@@ -1182,6 +1297,7 @@ rte_set_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
   }
 
   rfx=weed_to_rfx(filter,TRUE);
+  rfx->min_frames=-1;
   on_render_fx_pre_activate(NULL,rfx);
 
 }
@@ -1194,6 +1310,8 @@ void rte_set_defs_ok (GtkButton *button, lives_rfx_t *rfx) {
   int error;
   lives_colRGB24_t *rgbp;
 
+  if (mainw->textwidget_focus!=NULL) after_param_text_changed(mainw->textwidget_focus,rfx);
+
   if (rfx->num_params>0) {
     filter=weed_get_plantptr_value(rfx->source,"filter_class",&error);
     ptmpls=weed_get_plantptr_array(filter,"in_parameter_templates",&error);
@@ -1235,19 +1353,27 @@ void rte_set_defs_cancel (GtkButton *button, lives_rfx_t *rfx) {
 
 void load_default_keymap(void) {
   // called on startup
-  const gchar *keymap_file=g_strdup_printf("%s/%s/default.keymap",capable->home_dir,LIVES_CONFIG_DIR);
-  const gchar *keymap_template=g_strdup_printf("%s%sdefault.keymap",prefs->prefix_dir,DATA_DIR);
-  gchar *com;
+  gchar *keymap_file=g_strdup_printf("%s/%sdefault.keymap",capable->home_dir,LIVES_CONFIG_DIR);
+  gchar *keymap_template=g_strdup_printf("%s%sdefault.keymap",prefs->prefix_dir,DATA_DIR);
+  gchar *com,*tmp;
 
+  pthread_mutex_lock(&mainw->gtk_mutex);
   if (!g_file_test (keymap_file, G_FILE_TEST_EXISTS)) {
     com=g_strdup_printf("/bin/cp %s %s",keymap_template,keymap_file);
-    system(com);
+    dummyvar=system(com);
     g_free(com);
   }
   if (!g_file_test (keymap_file, G_FILE_TEST_EXISTS)) {
     // give up
-    d_print(g_strdup_printf(_("Unable to create default keymap file: %s\nPlease make sure your home directory is writable.\n"),keymap_file));
+    d_print((tmp=g_strdup_printf(_("Unable to create default keymap file: %s\nPlease make sure your home directory is writable.\n"),keymap_file)));
+    g_free(tmp);
+    g_free(keymap_file);
+    g_free(keymap_template);
+    pthread_mutex_unlock(&mainw->gtk_mutex);
     return;
   }
   on_load_keymap_clicked(NULL,NULL);
+  g_free(keymap_file);
+  g_free(keymap_template);
+  pthread_mutex_unlock(&mainw->gtk_mutex);
 }
